;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "render.h.s"
.include "collision.h.s"
.include "enemy.h.s"
.include "obstacle.h.s"


m_collision_acc: .db #0
m_spawnX: .db #0            ;;Auxilar variables to calculate collisions with ALL entities
m_spawnW: .db #0            ;;\ç
m_spawnH: .db #0


collision_GetSpawningX:

    ld  a, (m_spawnX)

ret

;;==============================================================
;; MAP_DRAW: Scrolls map, 
;; OUTPUT:
;;       A -> Collision state (0 = no collision, 1 = collision)
;; DESTROYS:
;;      AF, BC
;;===============================================================

collision_CheckEntitiesCollision:
    ;;
    ;;Check collision in X axis
    ;;
    call check_entities_collision_x

    ;;
    ;;Make if statement
    ;;
    jr z, no_collision
    jp m, no_collision

    ;;
    ;;If i collide in x, then i check y
    ;;
    call check_entities_collision_y
    
    jr z, no_collision
    jp m, no_collision

    
    ;;
    ;;If there is collision
    ;;
    ld  a, #1
    ret
    
    no_collision:
    ld  a, #0

ret

;;====================================================
;; CHECK_ENTITIES_COLLISION_X: Check colision in X
;; INPUTS: IX -> Entity 1
;;         IY -> Entity 2
;; DESTROYS A, BC
;;====================================================

check_entities_collision_x:

    ld a, de_x(ix)   ;;A = x of the first entity
    ld b, de_w(ix)   ;;B = width of the first entity
    ld c, de_x(iy)   ;;C = x of the second entity
    
    add b           ;;A = x + width
    add #1          ;;Vel. Hero X
    sub c           ;;A = (Result) - c

    jr z, purge
    jp m, purge

 
    ld a, de_x(iy)   ;; A = x of the second entity
    ld b, de_w(iy)   ;; B = width of the second entity
    ld c, de_x(ix)
    
    add b
    add #1          ;;Vel. Hero X
    sub c 

    ;;I do not make any checks because it will be handled by the first function

    purge:
    
ret

;;======================================================
;; CHECK_ENTITIES_COLLISION_X: Check colision in X
;; INPUTS: IX -> Entity 1
;;         IY -> Entity 2
;; DESTROYS A, BC
;;======================================================

check_entities_collision_y:

    ld a, de_y(ix)   ;;A = x of the first entity
    ld b, de_h(ix)   ;;B = width of the first entity
    ld c, de_y(iy)   ;;C = x of the second entity
    
    add b           ;;A = x + width
    add #4          ;;Vel. Hero Y
    sub c           ;;A = (Result) - c

    jr z, purge2
    jp m, purge2


    ld a, de_y(iy)   ;; A = x of the second entity
    ld b, de_h(iy)   ;; B = width of the second entity
    ld c, de_y(ix)   ;; C = x of the first entity

    add b
    add #4          ;;Vel. Hero Y
    sub c 

    ;;I do not make any checks because it will be handled by the first function

    purge2:
    
ret

;;==============================================================================
;; collision_CheckCollWithinEntities: Check if there's collision within any 
;; INPUTS: 
;;      A -> X position of the new entity to spawn
;;      D -> Width of the new entity to spawn
;;      E -> Height of the "" ""
;;==============================================================================

collision_CheckCollWithinEntities:

    ld  (m_spawnX), a                  ;;m_spawnX = A          
    ld  a, d                           ;;m_spawnW = D
    ld  (m_spawnW), a                  ;;\
    ld  a, e                           ;;m_spawnH = E
    ld  (m_spawnH), a                  ;;\
    ld  a, #0                          ;; Collision ACC = 0
    ld  (m_collision_acc), a           ;;\

    ld  hl, #collision_CheckSpawnEntitiesCollision
    call enemy_DoForAll

    ld  hl, #collision_CheckSpawnEntitiesCollision
    call obstacle_DoForAll

    ld  a, (m_collision_acc)
    cp  #0
    jr  z, no_entities_collision

        ld  a, #1
        ret

no_entities_collision:
ld  a, #0
ret


;;=======================================================================================
;; collision_CheckSpawnEntitiesCollision: check collisions btw spawning entity and ALL
;; INPUTS:
;;      
;; OUTPUT:
;;       A -> Collision state (0 = no collision, 1 = collision)
;; DESTROYS:
;;      AF, BC
;;=======================================================================================

collision_CheckSpawnEntitiesCollision:
    ;;
    ;;Check collision in X axis
    ;;
    call check_spawn_entities_collision_x

    ;;
    ;;Make if statement
    ;;
    jr z, no_spawn_collision
    jp m, no_spawn_collision

    ;;
    ;;If i collide in x, then i check y
    ;;
    call check_spawn_entities_collision_y
    
    jr z, no_spawn_collision
    jp m, no_spawn_collision

    
    ;;
    ;;If there is collision
    ;;
    ld  hl, #m_collision_acc
    inc (hl)
    ret
    
    no_spawn_collision:
ret

;;====================================================
;; CHECK_ENTITIES_COLLISION_X: Check colision in X
;; INPUTS: IX -> Entity 1
;;         IY -> Entity 2
;; DESTROYS A, BC
;;====================================================

check_spawn_entities_collision_x:

    ld  a, (m_spawnW)
    ld b, a             ;;B = width of the spawning entity
    ld  a, (m_spawnX)   ;;A = X of the spawing entity
    ld c, de_x(ix)      ;;C = x of the second entity
    
    add b           ;;A = x + width
    add #1          ;;Vel. Hero X
    sub c           ;;A = (Result) - c

    jr z, spawn_purge
    jp m, spawn_purge

    ld  a, (m_spawnX)
    ld  c, a
    ld  a, de_x(ix)   ;; A = x of the second entity
    ld  b, de_w(ix)   ;; B = width of the second entity
    
    add b
    add #1          ;;Vel. Hero X
    sub c 

    ;;I do not make any checks because it will be handled by the first function

    spawn_purge:
    
ret


;;======================================================
;; CHECK_ENTITIES_COLLISION_X: Check colision in X
;; INPUTS: IX -> Entity 1
;;         IY -> Entity 2
;; DESTROYS A, BC
;;======================================================

check_spawn_entities_collision_y:

    ld  a, (m_spawnH)
    ld  b, a
    ld a, #0
    ld c, de_y(ix)   ;;C = x of the second entity
    
    add b           ;;A = x + width
    add #4          ;;Vel. Hero Y
    sub c           ;;A = (Result) - c

    jr z, spawn_purge2
    jp m, spawn_purge2

    ld a, de_y(ix)   ;; A = x of the second entity
    ld b, de_h(ix)   ;; B = width of the second entity
    ld c, #0

    add b
    add #4          ;;Vel. Hero Y
    sub c 

    ;;I do not make any checks because it will be handled by the first function

    spawn_purge2:
    
ret