;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "main.h.s"
.include "dobleBuffer.h.s"

m_frontBuffer: .db #0xC0 ;;
m_backBuffer:  .db #0x80 ;; (We always draw on this buffer)

;;================================================
;;  BUFFER_SWITCH: Exchanges the values of the buffers and 
;;  changes the video memory pointer value to the 
;;  INPUTS: 
;;  DESTROYS: AF, BC
;;================================================
buffer_switch:
    ;;
    ;; Exchanges the value of the buffers
    ;;
    ld a, (m_backBuffer)            
    ld b, a                         
    ld a, (m_frontBuffer)           
    ld (m_backBuffer), a            
    ld a, b                         
    ld (m_frontBuffer), a           

    ;;
    ;; Move bits of B one position to the right.
    ;; 
    srl b                          
    srl b

    ;;
    ;; Change the video memory pointer value
    ;;
    ld l, b                        
    jp cpct_setVideoMemoryPage_asm  


;;================================================
;; BUFFER_CLEARBACK: Clear the back buffer 
;; INPUTS: 
;; DESTROYS: AF, HL, DE
;;================================================

buffer_clearBack:
    ;;
    ;; HL-> Back buffer
    ;;
    ld a, (m_backBuffer) 
    ld h, a  
    ld de, #0
    ld l, e

    ;;
    ;; BC-> Size
    ;;
    ld bc, #0x4000       ;;Carge in BC the ammount of memory that we are going to clean
    call cpct_memset_f8_asm

ret

buffer_clearFront:
    ;;
    ;; HL-> Back buffer
    ;;
    ld a, (m_frontBuffer) 
    ld h, a  
    ld de, #0
    ld l, e

    ;;
    ;; BC-> Size
    ;;
    ld bc, #0x4000       ;;Carge in BC the ammount of memory that we are going to clean
    call cpct_memset_f8_asm

ret

;;=================================================================================
;;  BUFFER_GETBACK: Charges into DE a pointer to the back buffer memory pointer
;;  INPUTS: 
;;  DESTROYS: DE, AF
;;  RETURN: DE -> 8000 (Back Buffer)
;;=================================================================================

buffer_getBack:
    ld a, (m_backBuffer) 
    ld d, a             
    ld e, #0             ;;DE = 8000
ret

;;==================================================================================
;;  BUFFER_GETFRONT: Charges into DE a pointer to the front buffer memory pointer
;;  INPUTS: 
;;  DESTROYS: DE, AF
;;  RETURN: DE -> C000 (Front Buffer)
;;==================================================================================

buffer_getFront:
    ld a, (m_frontBuffer)
    ld d, a
    ld e, #0             ;;DE = C000
ret

