;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;; INCLUDES
.include "main.h.s"
.include "enemy.h.s"
.include "bullet.h.s"
.include "player.h.s"
.include "dobleBuffer.h.s"
.include "render.h.s"
.include "entity.h.s"
.include "randomManager.h.s"
.include "obstacle.h.s"
.include "powerUp.h.s"
.include "menu.h.s"
.include "collision.h.s"
.include "enemyBullet.h.s"
.include "packs.h.s"


;;============================== CONSTANTS & VARIABLES =======================================;;

;; == Enemies == ;;
k_enemy_size = enemy_vector0_size
k_max_num_enemies = 10
m_num_enemies: .db 00                   ;;Counter to see how many entities we have created
m_next_enemy: .dw enemy_vector    ;;Pointer of the memory position of the next entity

;; == Clock enemies == ;;
max_clock  = 60  
clock:   .db #00                        ;;Clock to control the creation of enemies

;; == Aux Variables == ;;
enemy_mem: .dw 0x0000
enemy_aux: .db 0x00  

;; == Bounding Variables == ;;
bounding_x:.db #00 
bounding_y:.db #00 
bounding_w:.db #00 
bounding_h:.db #00 

;;===============================  MACROS =======================================;;

;;== Enemy vector ==;;
enemy_vector:
DefineNEnemy enemy_vector, k_max_num_enemies    ;;We are creating k_enemy_size default entities
DefineEnemy enemy1_data, 12, 0, 0x01, 0x00, 0x04, 0x10, enemyRight , 0x00, 0x00, 0x00, 0x00, enemy_CheckProximities, 0, 0, 3, 0  ;;Default data
DefineEnemy enemy2_data, 12, 0x10, 0x00, 0x00, 0x04, 0x10, enemyShooterRight , 0x00, 0x00, 0x00, 0x00, enemy_Shoot, 0, 0, 3, 10  ;;Default data
DefineEnemy enemy3_data, 12, 180, 0x01, 0x00, 0x04, 0x10, enemyRight , 0x00, 0x00, 0x00, 0x00, enemy_FollowHero, 0, 0, 1, 0  ;;Default data
DefineBoundingBox bounding_box, 0, 0, 0, 0  ;;Default data

;;==============================================================================
;; enemy_New: Register a new entity
;; INPUT: 
;;      DE -> Number of a specific active entities
;;      HL -> Pointer to the current last entity created
;       BC -> The size of the entity to create
;; DESTROYS: AF, BC, HL 
;; OUTPUT: HL -> Pointer to the current entity that we are going to create
;;==============================================================================

enemy_New:

    ;;
    ;;We add one more entity at the counter.
    ;;

    ld a, (m_num_enemies)               
    inc a
    ld (m_num_enemies), a

    ;;
    ;;We add the size of the entity to get the position of the next entity 
    ;;and we return the pointer to the current entity in HL
    ;;

    ld hl, (m_next_enemy)          
    ld bc, #k_enemy_size
    add hl, bc
    ld (m_next_enemy), hl
    or  a                          ;;Or of a to destroy the carry
    sbc hl, bc                     ;;Subtract HL - BC to get the pointer to the current entity

ret

;;====================================================================
;; ENT_COPY: Copy the values of one entity to another 
;; DESTROY: IX, IY
;; INPUT:   
;;     HL -> Origin entity
;;     DE -> Destinatio entity
;;====================================================================

enemy_Copy:

    ld (enemy_mem), de
    ld iy, (enemy_mem)

    ld  bc, #k_enemy_size         ;;Copy the size of the entity
    ldir

    call collision_GetSpawningX
    ld  de_x(iy), a

ret

;;====================================================================
;; ENT_COPY: Copy the values of one entity to another 
;; DESTROY: IX, IY
;; INPUT:   
;;     HL -> Origin entity
;;     DE -> Destinatio entity
;;====================================================================

enemy_CopyDefault:

    ld (enemy_mem), de
    ld iy, (enemy_mem)

    ld  bc, #k_enemy_size         ;;Copy the size of the entity
    ldir

ret

;;===========================================================
;; enemy_DoForAll: Update, draw and clear to all entities
;; INPUT:  HL-> Function 
;;===========================================================

enemy_DoForAll:

    ld a, (m_num_enemies)               ;;Loads the number of the entities 
    cp #0
    jr z, not_ent 
 
    ld ix, #enemy_vector                ;;We load the first entity of the vector in IX
    ld (method), hl                     ;;Put the function in method
    
    loop:
        push af                         ;;Push a on the stack to not lose them

        ld a, en_dead(ix)
        cp #1
        jr z, dead

        method = . + 1                  ;;Current position + 1 = the function of the next call
        call enemy_CheckProximities         ;;We call the method we have in hl 

        dead:

        pop af

        ;;
        ;;We put ix pointing to the next entity, 
        ;;so that the next time I draw it with that entity
        ;;
        ld bc, #k_enemy_size
        add ix, bc
        dec a 
    jr nz, loop
not_ent:
ret

;;============================================================
;; enemy_DESTROY: Deletes a bullet from the memory
;; INPUT:   
;;    IX -> Pointer to THIS bullet
;; DESTROYS:
;;    AF, HL, DE, BC
;;============================================================

enemy_Destroy:
    call ren_Clear_Front_Enemies
    ld d, de_x(iy)
    ld e, de_y(iy)
    ld l, d
    call pu_TryToSpawnPU
    call enemy_TurnBackNextEnemy            ;;M_NEXT_ENEMY one position back
    call enemy_DecreaseNumEnemys            ;;M_NUM_ENEMIES--
    
    push iy
    pop de

    ld  a, h                                ;;Check if the last enemy is the same as THIS
    cp  d                                   ;;|
    jr nz, make_copy                        ;;|

    ld  a, l                                ;;|
    cp  e                                   ;;|
    jr nz, make_copy                        ;;|

    call enemy_SetDeadState                 ;;In other case, don't make a Copy,
    ret	                                    ;;it's the same direction
                                            ;;only set this position (the last) to dead
    make_copy:

    ld  bc, #k_enemy_size                   ;;\Copy the last enemy of the vector
    ldir                                    ;;\to the position of THIS enemy that we want to destroy
    call enemy_SetDeadState                 
    
ret

;;=======================================================================
;; enemy_TURNBACKNEXTBULLET: Set back next bullet pointer
;; DETROY: 
;;      AF, HL, BC 
;;=======================================================================

enemy_TurnBackNextEnemy:

    ld  hl, (m_next_enemy)                 ;;HL = Pointer to the last active enemy                                                          
    ld  bc, #k_enemy_size                  
    or a 
    sbc hl, bc
    ld  (m_next_enemy), hl  

ret

;;=======================================================================
;; enemy_DECREASENUMenemyS: Decrease the number of enemys 
;; DETROY: 
;;      AF 
;;=======================================================================

enemy_DecreaseNumEnemys:

    ld  a, (m_num_enemies)                   ;;A = number of active enemys
    dec a                                   ;;A --
    ld  (m_num_enemies), a                   ;;<>
ret

;;=======================================================================
;; enemy_SETDEADSTATE: Set the last enemy's dead state to 1
;; DETROY: 
;;      AF
;;=======================================================================

enemy_SetDeadState:
    push ix

        ld ix, (m_next_enemy)
        ld a, #1
        ld en_dead(ix), a
        
    pop ix 
ret


;;===============================================================
;; ENEMY_HIT: Decrease in bullet_damage the health of the Enemy
;; INPUT:
;;      IY -> Pointer to enemy 
;;       D -> Bullet's damage 
;; DESTROYS:
;;       A, HL, BC
;;===============================================================

enemy_Hit:

    ld hl, #enemy_FollowHero
    ld   en_up_h(iy), h
    ld   en_up_l(iy), l

    ld  a, en_health(iy)        ;;A = Enemy's health
    cp  d
    jp  m, enemy_Destroy        ;;If 
    jp  z, enemy_Destroy

        sub d
        ld  en_health(iy), a
        ret

;;===============================================================
;; ENEMY_CheckProximities: Check everything is close
;; DESTROYS:
;;       A, HL, BC
;;===============================================================

enemy_CheckProximities:

        ld hl, #obstacle_Collisions
        call obstacle_EveryObstacle
        ld    iy, #player_data
        call enemy_CheckCollision

        ;;=======Check if the hero is close==========
        call enemy_CheckProximitiesWithHero     
        call enemy_CheckVision
        
        jp enemy_Wait4Hero  ;;HERO IS FAR -> continue walking


;;===============================================================
;; ENEMY_CheckProximitiesWithHero: Check if the hero is close to attack
;; DESTROYS:
;;       A, HL, BC
;;===============================================================

enemy_CheckProximitiesWithHero:

        ;;
        ;;Check proximity down
        ;;
        ld l, de_y(iy)  ;Position hero (HL)
        ld c, de_y(ix)  ;Position enemy (BC)
        ld a, #80       ;Range 

        call proximity
        cp #0
        jr z, hero_is_far

        ;;
        ;;Check proximity right
        ;;
        ld l, de_x(iy)  ;Position hero (HL)
        ld c, de_x(ix)  ;Position enemy (BC)
        ld a, #20       ;Range 

        call proximity
        cp #0
        jr z, hero_is_far

        ;;
        ;;Check proximity left
        ;;
        ld l, de_x(ix)  ;Position enemy (HL)
        ld c, de_x(iy)  ;Position hero (BC)
        ld a, #20       ;Range 
        
        call proximity
        cp #0
        jr z, hero_is_far

        ret

    hero_is_far:
    pop af
    ld  en_vy(ix), #0
    jp enemy_Wait4Hero  ;;HERO IS FAR -> continue walking

ret

;;=============================================
;; check proximity method
;;input: l -> position of hero
;;       c -> position of enemy
;;       a -> range of proximity
;;==============================================

proximity:

    ld h, #0x00
    ld b, #0x00
    add a, c            ;;Add BC(position enemy) + Range 
    ld c, a      
    
    or a
    sbc hl, bc          ;;substract the hero's position minus enemy's position + range
    jp m, continue      ;;if the result is negative, the hero is too close and the enemy attack
    
    ld a, #0

    continue:
ret             ;;the hero is nearby, back to continue checking proximities


;;===============================================================
;; ENEMY_CHECKVISION: Check if the hero is hidden behind an obstacle
;; DESTROYS:
;;       A, C
;;===============================================================

enemy_CheckVision:

    ld a, en_vx(ix)        ;; VEL X ENEMY
    cp #1
    jr z, looking_right_side

    looking_left_side:
        ld a, e_x(ix)       ;;POS X ENEMY
        ld c, e_x(iy)       ;;POS X HERO
        cp c
        jp m, hidden        ;;if negative, enemy cant see the hero
        jr z, hidden

        call enemy_SetBoxVisionLFT 
        
        jp cbb

    looking_right_side:
        ld a, e_x(iy)       ;;POS X HERO
        ld c, e_x(ix)       ;;POS X ENEMY
        cp c
        jp m, hidden        ;;if negative, enemy cant see the hero
        jr z, hidden

        call enemy_SetBoxVisionRGT

    cbb:

        call enemy_CreateBoundingBox
        push ix
            ld ix, #bounding_box
            ld hl, #obstacle_BoundingBoxCollisions
            call obstacle_EveryObstacle
        pop ix

        cp #5               
        jr z, hidden  ;;if exist collision between the bb and an obstacle, hero is hidden

        ;;hero is not hidden, enemy attacks him
        pop af
        
        ld hl, #enemy_FollowHero
        ld   en_up_h(ix), h
        ld   en_up_l(ix), l
     
     hidden: 
ret

;;===============================================================
;; ENEMY_SetBoxVisionRGT: Set values of Bounding Box (right side)
;; DESTROYS:
;;       A, C
;;===============================================================

enemy_CreateBoundingBox:
    push ix

        ld ix, #bounding_box

        ;;===POS X====;;
        ld a, (bounding_x)
        ld bb_x(ix), a

        ;;===POS Y====;;
        ld a, (bounding_y)
        ld bb_y(ix), a

        ;;===WIDTH====;;
        ld a, (bounding_w)
        ld bb_w(ix), a

        ;;===HEIGHT====;;
        ld a, (bounding_h)
        ld bb_h(ix), a

        ;call ent_Draw
    
    pop ix
ret


;;===============================================================
;; ENEMY_SetBoxVisionRGT: Set values of Bounding Box (right side)
;; DESTROYS:
;;       A, C
;;===============================================================

enemy_SetBoxVisionRGT:
    ;; POS X BB = POS X ENEMY
    ld a, e_x(ix)
    ld (bounding_x), a      

    ;; WIDTH BB = POS X HERO - POS X ENEMY
    ld a, e_x(iy)        ;;POS X HERO
    ld c, e_x(ix)        ;;POS X ENEMY
    sub c
    ld (bounding_w), a
    
    ;;======CHECK IF HERO IS UP OR DOWN==========

    ld a, e_y(ix)        ;; POS Y ENEMY
    ld c, e_y(iy)        ;; POS Y HERO
    cp c
    jp m, hero_is_downR
    jr z, hero_is_downR

    hero_is_upR:
        
        call enemy_SetBoxVisionUP

    ret

    hero_is_downR:

        call enemy_SetBoxVisionDWN
           
ret

;;===============================================================
;; ENEMY_SetBoxVisionLFT: Set values of Bounding Box (left side)
;; DESTROYS:
;;       A, C
;;===============================================================

enemy_SetBoxVisionLFT:
    ;; POS X BB = POS X HERO
    ld a, e_x(iy)
    ld (bounding_x), a      

    ;; WIDTH BB = POS X ENEMY - POS X HERO
    ld a, e_x(ix)        ;;POS X ENEMY
    ld c, e_x(iy)        ;;POS X HERO
    sub c
    ld (bounding_w), a
    
    ;;======CHECK IF HERO IS UP OR DOWN==========

    ld a, e_y(ix)        ;; POS Y ENEMY
    ld c, e_y(iy)        ;; POS Y HERO
    cp c
    jp m, hero_is_downL
    jr z, hero_is_downL

    hero_is_upL:
        
        call enemy_SetBoxVisionUP

    ret

    hero_is_downL:

        call enemy_SetBoxVisionDWN
           
ret

;;===============================================================
;; ENEMY_SetBoxVisionUP: Set values of Bounding Box (hero is up)
;; DESTROYS:
;;       A, C
;;===============================================================

enemy_SetBoxVisionUP:
    ;; POS Y BB = POS Y ENEMY
    ld a, e_y(iy)     ;;POS Y HERO
    ld (bounding_y), a

    ;; HEIGHT BB = POS Y ENEMY - POS Y ENEMY
    ld a, e_y(ix)        ;;POS Y ENEMY
    ld c, e_y(iy)        ;;POS Y HERO
    sub c
    ld (bounding_h), a
ret

;;===============================================================
;; ENEMY_SetBoxVisionDWN: Set values of Bounding Box (hero is down)
;; DESTROYS:
;;       A, C
;;===============================================================

enemy_SetBoxVisionDWN:
     ;; POS Y BB = POS Y ENEMY
     ld a, e_y(ix)     ;;POS Y ENEMY
     ld (bounding_y), a

     ;; HEIGHT BB = POS Y HERO - POS Y ENEMY
     ld a, e_y(iy)        ;;POS Y HERO
     ld c, e_y(ix)        ;;POS Y ENEMY
     sub c
     ld (bounding_h), a
ret

;;==========================================================
;; enemy_WAIT4HERO: The enemy waits for 
;; the hero moving from left to right
;; INPUT: 
;;      IX -> Pointer to entity (enemy)
;; DESTROY: 
;;      IY, A, BC, HL
;;===========================================================

enemy_Wait4Hero:

    ld a, en_vx(ix)
    cp #0
    jr nz, already_has_velocity
        ld en_vx(ix), #1

    already_has_velocity:

    ;;
    ;;Walk from left to right
    ;;
    walk:

        ;;check right wall
        ld a, de_x(ix)
        cp #80-4
        jr z, move_to_left
        jp p, move_to_left

        ;;check obstacle right
        ld a, en_bm(ix)
        cp #1
        jr z, move_to_left
        
        ;;check left wall
        ld a, de_x(ix)
        cp #16
        jr z, move_to_right
        jp m, move_to_right

        ;;check obstacle left
        ld a, en_bm(ix)
        cp #2
        jr z, move_to_right

        jp ent_Move

        ;;change direction on the left
        move_to_left:
        ld hl, #enemyLeft
        ld de_sprite_l(ix), l
        ld de_sprite_h(ix), h
        ld en_vx(ix), #-1
        jp ent_Move

        ;;change direction on the right
        move_to_right:
        ld hl, #enemyRight
        ld de_sprite_l(ix), l
        ld de_sprite_h(ix), h
        ld en_vx(ix), #1
        jp ent_Move


;;=============================================================================
;; ENEMY_FOLLOWHERO: The enemy will follow the entire screen to the character
;; REGISTROS DESTRUIDOS: IY, HL, BC, A
;; ENTRADA: IX -> Puntero a entidad
;;============================================================================

enemy_FollowHero:
    
    ld hl, #obstacle_Collisions
    call obstacle_EveryObstacle
    ld    iy, #player_data
    call enemy_CheckCollision

    ld en_vx(ix), #0x00
    ld en_vy(ix), #0x00


    ;;
    ;;Compare the position of the enemy and the hero on the x-axis
    ;;

    check_x:

        ld h, #0x00
        ld l, de_x(iy)       ;;Position hero (HL)
        ld b, #0x00
        ld c, de_x(ix)       ;;Position enemy (BC)

        or a
        sbc hl, bc          ;;We subtract the positions

        jr z, check_y
        jp m, to_Left       ;;If the result is negative, the enemy will move to the left
                            ;;otherwise, it will move to the right

        to_Right:
            ;; == Check collisions with obstacles == ;;
            ld a, en_bm(ix)
            cp #1
            jr z, check_y
            ;; ===================================== ;;

            ld en_vx(ix), #1
            jr check_y
        
        to_Left:
            ;; == Check collisions with obstacles == ;;
            ld a, en_bm(ix)
            cp #2
            jr z, check_y
            ;; ===================================== ;;

            ld en_vx(ix), #-1
            jr check_y

    ;;
    ;;Compare the position of the enemy and the hero on the y-axis
    ;;

    check_y:

        ld h, #0x00
        ld l, de_y(iy)       ;;Position hero (HL)
        ld b, #0x00
        ld c, de_y(ix)       ;;Position enemy (BC)

        or a
        sbc hl, bc          ;;We subtract the positions

        jr z, finish
        jp m, to_Up         ;;If the result is negative, the enemy will move up
                            ;;otherwise, it will move to the down

        to_Down:
            ;; == Check collisions with obstacles == ;;
            ld a, en_bm(ix)
            cp #3
            jr z, finish
            ;; ===================================== ;;

            ;; == Change spite ==
            ld hl, #0xFFFF
            ld de_sprite_l(ix), l
            ld de_sprite_h(ix), h

            ld en_vy(ix), #4
            jr finish

        to_Up:
            ;; == Check collisions with obstacles == ;;
            ld a, en_bm(ix)
            cp #4
            jr z, finish
            ;; ===================================== ;;

            ld en_vy(ix), #-4
            jr finish

    
    finish:
        call enemy_ChangeSprite
        jp ent_Move



;;============================================================================
;; enemy_Shoot: Creates a bullet on the Enemy position
;; DESTROYS:
;;       AF
;;============================================================================

enemy_Shoot:
    call enemy_DirectionShooting

    ld a, en_delay(ix)
    dec a
    ld en_delay(ix), a
    cp #0
    jr z, shoott
    jp not_fire

    shoott:
    ld a , #10
    ld en_delay(ix), a

    call bulletEn_New
    call bulletEn_Copy
    
    ;;====set position=====
    ld a, de_x(ix)
    ld de_x(iy), a
    ld a, de_y(ix)
    add #4
    ld de_y(iy), a


    ;;====set velocity=====
    ld a, (enemy_aux)
    ld  b_vx(iy), a
    ld a, #0
    ld  b_vy(iy), a

not_fire:
ret

enemy_DirectionShooting:
    ld a, de_x(ix)
    cp #40
    jp m, shoot_right

    shoot_left:
        ld a, #-2
        ld (enemy_aux), a

        ld hl, #enemyShooterLeft
        ld de_sprite_l(ix), l
        ld de_sprite_h(ix), h

        ret
    shoot_right:
        ld a, #2
        ld (enemy_aux), a

        ld hl, #enemyShooterRight
        ld de_sprite_l(ix), l
        ld de_sprite_h(ix), h

        ret
        
;;========================================================
;; ENEMY_CHANGESPRITE: Set the correct sprite 
;; DESTROY -> A, HL, DE
;;========================================================
enemy_ChangeSprite: 
    ;;
    ;;Right
    ;;
    sprite1:
        ld a, en_vx(ix)
        cp #1
        jr nz, sprite2
        ld a, en_vy(ix)
        cp #0
        jr nz, sprite2
            ld hl, #enemyRightAlert
            jp setSprite
    ;;
    ;;Left
    ;;
    sprite2:
        ld a, en_vx(ix)
        cp #-1
        jr nz, sprite3
        ld a, en_vy(ix)
        cp #0
        jr nz, sprite3
            ld hl, #enemyLeftAlert
            jp setSprite
    ;;
    ;;Up
    ;;
    sprite3:
        ld a, en_vx(ix)
        cp #0
        jr nz, sprite4
        ld a, en_vy(ix)
        cp #-4
        jr nz, sprite4
            ld hl, #enemyUp
            jp setSprite

    ;;
    ;;Down
    ;;
    sprite4:
        ld a, en_vx(ix)
        cp #0
        jr nz, sprite5
        ld a, en_vy(ix)
        cp #4
        jr nz, sprite5
            ld hl, #enemyDown
            jp setSprite
    ;;
    ;;Up right
    ;;
    sprite5:
        ld a, en_vx(ix)
        cp #1
        jr nz, sprite6
        ld a, p_vy(ix)
        cp #-4
        jr nz, sprite6
            ld hl, #enemyUpRight
            jp setSprite
    ;;
    ;;Up left
    ;;
    sprite6:
        ld a, en_vx(ix)
        cp #-1
        jr nz, sprite7
        ld a, en_vy(ix)
        cp #-4
        jr nz, sprite7
            ld hl, #enemyUpLeft
            jp setSprite
        
    ;;
    ;;Down right
    ;;
    sprite7:
        ld a, en_vx(ix)
        cp #1
        jr nz, sprite8
        ld a, en_vy(ix)
        cp #4
        jr nz, sprite8
            ld hl, #enemyDownRight
            jp setSprite

    ;;
    ;;Down left
    ;;
    sprite8:
        ld a, en_vx(ix)
        cp #-1
        jr nz, exit_sprite
        ld a, en_vy(ix)
        cp #4
        jr nz, exit_sprite
            ld hl, #enemyDownLeft
            jp setSprite
        ret

    setSprite:
        ld de_sprite_l(ix), l
        ld de_sprite_h(ix), h
    ;;Check 
    exit_sprite:
ret

;;========================================================
;; ENEMY_CREATENEWENEMY: Create new enemies 
;; in the array of entities with the keyboard
;; DESTROY -> A, HL, DE
;;========================================================

enemy_CreateNewEnemy:
  
  ;;
  ;;If the number of entities the vector is equal to the maximum of entities,
  ;;no more enemies are created
  ;;
  ld a, (m_num_enemies)    
  cp #k_max_num_enemies       
  jr z, no_more_enemies

  ;;
  ;;Enemies of type 1 that are created with the key M
  ;;

  ld a, (spawn_enemies)
  cp #1
  jr nz, no_more_enemies               ;;if it is not pressed, enemies are not created
       ;;
       ;;Load and increment the clock 
       ;;
       ld a, (clock)
       inc a
       ld (clock), a 

       ;;
       ;;Check if the clock is #max_clock
       ;;
       cp #max_clock
       jr nz, no_more_enemies

           ;;
           ;;If clock = #max_clock we create a new enemy and we reset the clock.
           ;;
           call enemy_New
           ex de, hl
           ld hl, #enemy3_data       ;;Create a new type 1 enemy
           call enemy_CopyDefault

           call enemy_New
           ex de, hl
           ld hl, #enemy3_data       ;;Create a new type 1 enemy
           call enemy_CopyDefault
           ld a, #65
           ld de_x(iy), a 

           ld a, #00                 ;;Restart the clock
           ld (clock), a

   no_more_enemies:
   
ret	

;;=======================================================================
;; ENEMY_EVERYENEMY: Iterate through all the enemies
;; INPUT:
;;    IY -> Pointer to the enemy's vector
;;    AF -> Number of active enemies
;;    BC -> Size of an enemy  
;;        
;;=======================================================================

enemy_EveryEnemy:

    ld   a, (m_num_enemies)                   ;;A = Number 
    ld  iy, #enemy_vector                    ;;IY = Pointer to the enemies vector
    cp #0
    jr z, not_dfe                   
    
    loop_dfe:
        push af                                 ;;Push a on the stack to not lose it

        ld a, en_dead(iy)
        cp #1
        jr z, dead2

        call bullet_Collissions                 ;;We call the method we have in hl 

        dead2:

        pop af

        ld  bc, #k_enemy_size
        add iy, bc                 
        dec a                                   ;;Next iteration
    jr nz, loop_dfe

not_dfe:
ret

;;===================================================
;; ENEMY_CHECKCOLLISION: Check the collision 
;; between the array of enemies and the hero
;; INPUT: IX -> ENEMY
;;        IY -> HERO
;;===================================================

enemy_CheckCollision:
    
    call collision_CheckEntitiesCollision

    cp #0                ;;NO COLLISION
    jr z, no_collision

    ;;COLLISION
    jp  player_Hit
    
    no_collision:
ret


;;================================================================              
;;  ENEMY_GETENEMYDATA: Returns a pointer to the enemies vector
;;  OUTPUTS
;;      IY -> Pointer to the enemy vector
;;  DESTROYS:
;;      IY
;;================================================================

enemy_GetEnemiesPtr:

    ld  iy, #enemy_vector

ret

;;==================================================================              
;;  ENEMY_GETENEMYSIZE: Returns the size of the enemies's vector
;;      BC -> Size of an enemy
;;  DESTROYS:
;;      BC
;;==================================================================

enemy_GetEnemySize:
    ld  bc, #k_enemy_size
ret 

;;==================================================================              
;;  ENEMY_GETENEMIESNUMB: Returns the number of active enemies
;;  OUTPUTS
;;       A -> Number of active enemies
;;  DESTROYS:
;;      AF
;;==================================================================

enemy_GetEnemiesNumb:

    ld   bc, #m_num_enemies

ret

;;=====================================================
;; enemy_DestroyEveryEnemy: destroy every enemy (captain obvious)
;; DESTROY: HL, A, BC
;;=====================================================

enemy_DestroyEveryEnemy:

    push ix
    ld hl, #enemy_DestroyAll
    call enemy_DoForAll
    pop ix

    ld a, #00
    ld (m_num_enemies), a        ;;Set number of enemies = 0
    ld hl, #enemy_vector      
    ld (m_next_enemy), hl        ;;Set next entity to the array's first position

ret

;;============================================================
;; enemy_DESTROY: Deletes a bullet from the memory
;; INPUT:   
;;    IX -> Pointer to THIS bullet
;; DESTROYS:
;;    AF, HL, DE, BC
;;============================================================

enemy_DestroyAll:
    call ren_Clear_FrontBack

    ld  a, (m_num_enemies)
    cp  #0
    jr  z, exit_destroy2

        call enemy_TurnBackNextEnemy            ;;M_NEXT_ENEMY one position back
        call enemy_DecreaseNumEnemys            ;;M_NUM_ENEMIES--
        
        push ix
        pop de                     
        
        ld  a, h                                ;;Check if the destination copy is the same as the origin
        cp  d                                   ;;|
        jr nz, make_copy2                       ;;|
                                                ;;\ If there's at least one, diference -> Make a memory movement
        ld  a, l                                ;;|
        cp  e                                   ;;|
        jr nz, make_copy2                       ;;|

        call enemy_SetDeadState                 ;;In other case, don't make a Copy,
        ret	                                    ;;it's the same direction
                                                ;;only set this position (the last) to dead
        make_copy2:

        ld  bc, #k_enemy_size                   ;;\Copy the last enemy of the vector
        ldir                                    ;;\to the position of THIS enemy that we want to destroy
        call enemy_SetDeadState                 
    
        
exit_destroy2:
    
ret

enemy_MoveScroll:

    ld  a, #200-12             ;;A = 190 (Down side screen - (Height + VX of PU)     
    ld  b, de_y(ix)            ;;B = DE_Y (Y position of the PU)
    cp  b                      ;;A -= B
    jr  z, kill_enemy             ;;If z = 1, then destroy the bullet
    jp  c, kill_enemy            ;;(200 - 10) - DE_X -> C (Example: RESULT = FA = -6 = 250), so 

        ld a, de_y(ix)
        add #4
        ld de_y(ix), a
        ret

    kill_enemy:
    push ix
    pop iy 
    jp enemy_Destroy
    
ret

enemy_Spawn:

    ld  a,  #k_max_num_enemies
    push af
    ld  a, (m_num_enemies)
    ld  b, a
    inc b
    pop  af
    cp  b
    ret z

    call enemy_New
    ex de, hl
    ld  hl, #enemy1_data
    call enemy_Copy

ret


enemy_Spawn2:

    ld  a,  #k_max_num_enemies
    push af
    ld  a, (m_num_enemies)
    ld  b, a
    inc b
    pop  af
    cp  b
    ret z

    call enemy_New
    ex de, hl
    ld  hl, #enemy1_data
    call enemy_CopyDefault

ret

enemy2_Spawn2:

    ld  a,  #k_max_num_enemies
    push af
    ld  a, (m_num_enemies)
    ld  b, a
    inc b
    pop  af
    cp  b
    ret z

    call enemy_New
    ex de, hl
    ld  hl, #enemy2_data
    call enemy_CopyDefault

ret