;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "bullet.h.s"
.include "main.h.s"
.include "render.h.s"
.include "enemy.h.s"
.include "obstacle.h.s"
.include "powerUp.h.s"
.include "collision.h.s"
.include "entity.h.s"
.include "enemyBullet.h.s"
.include "player.h.s"

;;LOCAL VARIABLES

m_next_bulletEn: .dw bullet_enemyvector       ;;Pointer of the memory position of the next entity 
m_num_bulletEn: .db 00                   ;;Counter to see how many entities we have created
bullet_mem: .dw 0x0000

;; == Auxiliar Variables == ;;
m_bullet_counter: .db #0x00

;;============================== CONSTANTS =======================================;;

;;== Bullets Enemies ==;;
k_max_num_bulletsEn = 20               ;;Constant for the maximum number of bullets
k_bulletEn_size = bullet_enemyvector0_size       ;;Dynamic size of Bullet
m_bulletEn_lft = 30                         ;;Life cycles of a bullet

;;== Bullet vector ==;;
bullet_enemyvector:
DefineNBullets bullet_enemyvector, k_max_num_bulletsEn

;;== Default bullet for new instances, i.e. the default constructor ==;;
DefineBullet bulletEn_data, 0x00, 0x00, 0x01, 0x00, 0x01, 0x04, _bulletSprite, 0x00, 0x00, 0x00, 0x00, bulletEn_CheckLFT, m_bulletEn_lft, 0, #1

;;==============================================================================
;; BULLET_NEWENEMY: Register a new bullet
;;      INPUT: 
;;      DESTROYS: AF, BC, HL 
;;      OUTPUT: 
;;          HL -> Pointer to the current bullet that we are going to create
;;==============================================================================

bulletEn_New:

    ld  a, (m_num_bulletEn)
    inc a                          ;;A++
    ld (m_num_bulletEn), a           ;;<>

    ld hl, (m_next_bulletEn)         ;;HL = Pointer to the last active bullet    
    ld bc, #k_bulletEn_size          ;;BC = Bullet's size
    add hl, bc                     ;;HL += BC -> the position of the next new bullet
    ld (m_next_bulletEn), hl         ;;NEXT_ENTITY = HL (last active bullet)
    or  a                          ;;To put Carry to 0 (a or a = 0)
    sbc hl, bc                     ;;HL -= BC -> to get the pointer to the current entity

    ex de, hl

    ld  hl, #bulletEn_data       ;;Else -> Create a new bullet
    ret

ret

;;============================================================
;; BULLET_COPY: Copy the values of one bullet to another 
;; INPUT:
;;      HL -> Origin data to copy
;; DESTROY: 
;;      BC, IY
;; OUTPUT:
;;      IY -> Pointer to the new bullet/laser created
;;============================================================

bulletEn_Copy:
    ld (bullet_mem), de
    ld iy, (bullet_mem)            ;;IY = Pointer to the new bullet created
    ld  bc, #k_bulletEn_size         ;;Copy the size of the entity
    ldir 

ret

;;=======================================================================
;; BULLET_DOFORALL: Update, draw and clear to all bullets
;; INPUT:  HL-> Function 
;;=======================================================================

bulletEn_DoForAll:

    ld a, (m_num_bulletEn)           ;;Loads the number of the entities 
    cp #0
    jr z, not_dfa2  

    ld ix, #bullet_enemyvector          ;;Loads the first entity of the vector on IX
    ld (meth2), hl                ;;HL = function parameter
    
    loop2:

        push af                    ;;Push a on the stack to not lose it

        ld a, b_dead(ix)
        cp #1
        jr z, dead2

        meth2 = . + 1             ;;Current position + 1 = the function of the next call
        call ent_Move       ;;We call the method we have in hl 

        dead2:

        pop af

        ld bc, #k_bulletEn_size      ;;Move M_NEXT_BULLET pointer to the next position by the bullets size
        add ix, bc              
        dec a            
    jr nz, loop2                    ;;Next iteration

not_dfa2:
ret

;;============================================================
;; BULLET_DESTROY: Deletes a bullet from the memory
;; INPUT:   
;;    IX -> Pointer to THIS bullet
;; DESTROYS:
;;    AF, HL, DE, BC
;;============================================================

bulletEn_Destroy:
    call ren_Clear_FrontBack
    
    ld  a, (m_num_bulletEn)
    cp  #0
    jr  z, exit_destroy

        call bulletEn_TurnBackNextBullet
        call bulletEn_DecreaseNumBullets

        push ix
        pop de   

        ld  a, h                                ;;Check if the last bullet is the same as THIS
        cp  d                                   ;;|
        jr nz, make_copy                        ;;|

        ld  a, l                                ;;|
        cp  e                                   ;;|
        jr nz, make_copy                        ;;|

        call bulletEn_SetDeadState
        ret	

        make_copy:
        ld  bc, #k_bulletEn_size                  ;;Copy the last bullet of the vector
        ldir                                    ;;to the position of THIS bullet that we want to destroy
        call bulletEn_SetDeadState
        
exit_destroy:
ret

;;=======================================================================
;;BULLET_TURNBACKNEXTBULLET: Set back next bullet pointer
;; DETROY: AF, HL, BC 
;;=======================================================================

bulletEn_TurnBackNextBullet:

    ld  hl, (m_next_bulletEn)                 ;;HL = Pointer to the last active bullet                                                          
    ld  bc, #k_bulletEn_size                  
    or a 
    sbc hl, bc
    ld  (m_next_bulletEn), hl  

ret

;;=======================================================================
;;BULLET_DECREASENUMBULLETS: Decrease the number of bullets 
;; DETROY: AF 
;;=======================================================================

bulletEn_DecreaseNumBullets:

    ld  a, (m_num_bulletEn)                   ;;A = number of active bullets
    dec a                                   ;;A --
    ld  (m_num_bulletEn), a                   ;;<>
ret

;;=======================================================================
;;BULLET_SETDEADSTATE: Set the last bullet's dead state to 1
;; DETROY: A
;;=======================================================================

bulletEn_SetDeadState:
    push ix

        ld ix, (m_next_bulletEn)
        ld a, #1
        ld b_dead(ix), a
        
    pop ix 
ret

;;=====================================================================
;; bullet_DestroyEveryBullet: destroy every bullet (captain obvious)
;; DESTROY: HL, A
;;=====================================================================

bulletEn_DestroyEveryBullet:

    push ix
    ld hl, #bulletEn_Destroy
    call bulletEn_DoForAll
    pop ix

    ld a, #00
    ld (m_num_bulletEn), a        ;;Set number of bullets = 0
    ld hl, #bullet_enemyvector      
    ld (m_next_bulletEn), hl        ;;Set next entity to the array's first position

ret

;;===========================================================
;; BULLET_CHECKLFT (*): 
;;
;; (*) -> Tail recursion
;;===========================================================

bulletEn_CheckLFT:

    ld hl, #obstacle_BulletCollisions
    call obstacle_EveryObstacle
    call bulletEn_Collision
    call bullet_CheckScreenCollissions 

    ld   a, b_lft(ix)               ;;Check the lifetime time of ix bullet
    cp   #0                         ;;|
    jr   z, bullet_die              ;;\ If it's time to die, then stop updating

        dec a   
        ld  b_lft(ix), a
        call bullet_Move
        ret
        
    bullet_die:
    jp bulletEn_Destroy


;;=========================================================================================
;; ENEMY_CHECKCOLLISION: Check the collision between the array of enemies and the hero
;; INPUT: IX -> BULLET
;;        IY -> PLAYER
;;=========================================================================================

bulletEn_Collision:
    
    ld iy, #player_data

    call collision_CheckEntitiesCollision

    cp #0                ;;NO COLLISION
    jr z, no_collision

    ;;COLLISION
    ld  d, b_damage(ix)                 ;;B = bullet's damage
    call player_HitByBullet                      ;;Hit enemy with B damage
    
    no_collision:
ret

bulletEn_MoveScroll:

    ld  a, #200-12             ;;A = 190 (Down side screen - (Height + VX of PU)     
    ld  b, de_y(ix)            ;;B = DE_Y (Y position of the PU)
    cp  b                      ;;A -= B
    jr  z, dest_bullet             ;;If z = 1, then destroy the bullet
    jp  c, dest_bullet           ;;(200 - 10) - DE_X -> C (Example: RESULT = FA = -6 = 250), so 

        ld a, de_y(ix)
        add #4
        ld de_y(ix), a 
        ret

    dest_bullet:
    push ix
    pop iy 
    jp bulletEn_Destroy
    
ret

bulletEn_GetBulletSize:
    ld  bc, #k_bulletEn_size
ret	