;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "gui.h.s"
.include "main.h.s"
.include "player.h.s"
.include "map.h.s"
.include "dobleBuffer.h.s"
.include "render.h.s"
.include "packs.h.s"

lifes_aux_x: .db #4
guiLevelLabel: .dw #0xFFFF
arrowFlicker: .db #0
arrowFlickersCounter: .db #5
arrowFlickerTimer: .db #8
arrowDesactivated: .dw _gui_arrow
arrowActivated: .dw _gui_arrow2
arrowFlickerActivation: .db #0



gui_DrawLifes:


   ld  ix, #player_data
   ld   a, #1
   ld   (lifes_aux_x), a
   ld   a, p_lf(ix)

      loop_lifes:

        push af

        call buffer_getBack                ;; Charges into DE the back buffer memory pointer
        ld   a, (lifes_aux_x)
        ld   c, a
        ld   b, #10                        ;; B = Entity Y
        call cpct_getScreenPtr_asm

        ex   de, hl                        ;; DE = Pointer to memory
        ld   hl, #_player_life
        ld   b, #16                        ;; Height
        ld   c, #4                        ;; Width
        call cpct_drawSprite_asm

        call buffer_getFront                ;; Charges into DE the back buffer memory pointer
        ld   a, (lifes_aux_x)
        ld   c, a
        ld   b, #10                       ;; B = Entity Y
        call cpct_getScreenPtr_asm

        ex   de, hl                        ;; DE = Pointer to memory
        ld   hl, #_player_life
        ld   b, #16                        ;; Height
        ld   c, #4                        ;; Width
        call cpct_drawSprite_asm

        ld  a, (lifes_aux_x)
        add #5
        ld  (lifes_aux_x), a

        pop af

        dec a
        cp  #0
        jr  nz, loop_lifes
   
ret


gui_DrawLevel:

    call gui_DrawTopData
    call gui_DrawLifes
    call gui_DrawColours
    call gui_DrawLevelLabel
    call gui_DrawLevelPhoto

ret

gui_DrawLevelLabel:


   call buffer_getBack                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                       ;; C = Entity X
   ld   b, #104                      ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, (guiLevelLabel)
   ld   b, #32                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

   call buffer_getFront                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                       ;; C = Entity X
   ld   b, #104                      ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, (guiLevelLabel) 
   ld   b, #32                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

   ret 


gui_DrawColours:


   call buffer_getBack                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                       ;; C = Entity X
   ld   b, #72                     ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, #level_colours
   ld   b, #32                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

   call buffer_getFront                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                       ;; C = Entity X
   ld   b, #72                     ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, #level_colours
   ld   b, #32                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

ret

gui_InitColours:

   call buffer_getBack                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                       ;; C = Entity X
   ld   b, #72                     ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, #_colours_0
   ld   b, #32                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

   call buffer_getFront                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                       ;; C = Entity X
   ld   b, #72                     ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, #_colours_0
   ld   b, #32                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

ret

gui_UpdateColours:

    push hl
    push hl

   call buffer_getBack                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                       ;; C = Entity X
   ld   b, #72                     ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   pop hl
   ld   b, #32                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

   call buffer_getFront                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                       ;; C = Entity X
   ld   b, #72                     ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   pop hl 
   ld   b, #32                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

ret


gui_DrawTopData:

   call buffer_getBack                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                           ;; C = Entity X
   ld   b, #0                          ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, #_gui_top_data
   ld   b, #72                         ;; Height
   ld   c, #16                        ;; Width
   call cpct_drawSprite_asm

      call buffer_getFront               ;; Charges into DE the back buffer memory pointer
   ld   c, #0                           ;; C = Entity X
   ld   b, #0                        ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, #_gui_top_data
   ld   b, #72                         ;; Height
   ld   c, #16                        ;; Width
   call cpct_drawSprite_asm

   call buffer_getBack                ;; Charges into DE the back buffer memory pointer
   ld   c, #4                           ;; C = Entity X
   ld   b, #45                        ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, #_gui_arrow
   ld   b, #16                         ;; Height
   ld   c, #8                       ;; Width
   call cpct_drawSprite_asm

      call buffer_getFront               ;; Charges into DE the back buffer memory pointer
   ld   c, #4                          ;; C = Entity X
   ld   b, #45                        ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   ld   hl, #_gui_arrow
   ld   b, #16                       ;; Height
   ld   c, #8                       ;; Width
   call cpct_drawSprite_asm

   call gui_DrawLifes


ret


gui_DrawArrowFlicker:


push hl
push hl
   call buffer_getBack                ;; Charges into DE the back buffer memory pointer
   ld   c, #4                           ;; C = Entity X
   ld   b, #45                        ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   pop  hl
   ld   b, #16                         ;; Height
   ld   c, #8                       ;; Width
   call cpct_drawSprite_asm

   call buffer_getFront               ;; Charges into DE the back buffer memory pointer
   ld   c, #4                          ;; C = Entity X
   ld   b, #45                        ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                         ;; DE = Pointer to memory
   pop hl
   ld   b, #16                       ;; Height
   ld   c, #8                       ;; Width
   call cpct_drawSprite_asm


ret


gui_DrawLevelPhoto:

   call buffer_getBack                ;; Charges into DE the back buffer memory pointer
   ld   c, #0                         ;; C = Entity X
   ld   b, #136                       ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                             ; DE = Pointer to memory
   ld   hl, #level_photo
   ld   b, #64                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

   call buffer_getFront                 ;; Charges into DE the back buffer memory pointer
   ld   c, #0                           ;; C = Entity X
   ld   b, #136                         ;; B = Entity Y
   call cpct_getScreenPtr_asm

   ex   de, hl                          ;; DE = Pointer to memory
   ld   hl, #level_photo
   ld   b, #64                         ;; Height
   ld   c, #16                         ;; Width
   call cpct_drawSprite_asm

ret


gui_ArrowFlicker:

    ld  a, (arrowFlickerActivation)
    cp  #1
    jr  nz, exit_flickering

    ld  a, (arrowFlickersCounter)
    cp  #0
    jr  z, endOfFlickers

    ld  a, (arrowFlicker)
    cp  #0
    jr  z, black

    ld  a, (arrowFlickerTimer)
    dec a
    jr  z, changeFlicker


    ld  (arrowFlickerTimer), a

    ;;DIBUJAR;;
    ld  hl, (arrowDesactivated)
    call gui_DrawArrowFlicker
    jr  exit_flickering

    changeFlicker:
    ld  a, #0
    ld  (arrowFlicker), a
    ld  a, #8
    ld  (arrowFlickerTimer), a
    ld  a, (arrowFlickersCounter)
    dec a   
    ld  (arrowFlickersCounter), a
    jr  exit_flickering

    black:

    ld  a, (arrowFlickerTimer)
    dec a
    jr  z, changeFlicker2

    ld  (arrowFlickerTimer), a

    ;;DIBUJAR;;
    ld  hl, (arrowActivated)
    call gui_DrawArrowFlicker
    jr  exit_flickering

    changeFlicker2:
    ld  a, #1
    ld  (arrowFlicker), a
    ld  a, #8
    ld  (arrowFlickerTimer), a
    ld  a, (arrowFlickersCounter)
    dec a   
    ld  (arrowFlickersCounter), a
    jr  exit_flickering

    endOfFlickers:
    ld  a, #5
    ld  (arrowFlickersCounter), a
    ld  a, #0
    ld  (arrowFlickerActivation), a
    ld  a, #8
    ld  (arrowFlickerTimer), a

    ld hl, (arrowDesactivated)
    call gui_DrawArrowFlicker

exit_flickering:
  
ret
