;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

;; Include all CPCtelera constant definitions, macros and variables
.include "main.h.s"
.include "dobleBuffer.h.s"
.include "entity.h.s"
.include "player.h.s"
.include "bullet.h.s"
.include "render.h.s"
.include "enemy.h.s"
.include "obstacle.h.s"
.include "powerUp.h.s"
.include "menu.h.s"
.include "deadScreen.h.s"
.include "map.h.s"
.include "enemyBullet.h.s"
.include "gui.h.s"
.include "packs.h.s"
.include "winner.h.s"
.include "sound.h.s"

.area _DATA
.area _CODE

m_isPlayerDead: .db #0x00

isr_var: .db #0
framerate_value: .db #16
inGame: .db #0

;;=================================
;; MAX FRAMERATE CONTROL
;;=================================

isr:
    ld a, (isr_var)
    cp #0
    jp z, reset
    dec a
    ld (isr_var), a

    ld a, (inGame)
    cp #0
    ret nz

    call playMusic
    ret

    reset:
    ld a, (framerate_value)
    ld (isr_var), a

    ld a, (inGame)
    cp #0
    ret nz
    call playMusic
ret

waitISR:

    wait_loop:

        ld a, (isr_var)
        cp #0
        ret z

    jr wait_loop

ret

;;============================
;; MAIN_INIT: Initialize main and create the FIRST ENEMY
;;============================

main_init:

	;;
    ;; Disable firmware to prevent it from interfering with string drawing
	;;
    call cpct_disableFirmware_asm
	
	;;
    ;;Set video mode 0
	;;
    ld    c, #0
    call cpct_setVideoMode_asm 

    ;;
    ;;Set black border
    ;;
    ld l, #0x10
    ld h, #0x14
    call cpct_setPALColour_asm

    ;;
    ;; Set ISR
    ;;
    ld hl, #isr
    call cpct_setInterruptHandler_asm
    
    ;;
    ;;Set palette
	;;
    ld hl, #_palette
    ld de, #16
    call cpct_setPalette_asm

    call loadLevelMusic
    call loadSoundEffects
    
ret




;;============================
;; _MAIN
;;============================

_main::
    ;;
    ;; Change the position stack
    ;;
    ld sp, #0x8000

    call main_init

        menu_init:
            ld a, #0
            ld (inGame), a
            call stopMusic
            call loadLevelMusic

        menu_loop:
            ;;Decompress level1
            ld hl, #_level1_pack_end
            ld de, #descompress_buffer_end
            call cpct_zx7b_decrunch_s_asm

            ld hl, #guiLevel1
            ld (guiLevelLabel), hl

            call menu_StartGame
            call cpct_waitVSYNC_asm

        jr menu_loop

        win_loop:

            call winner_WinState
            call cpct_waitVSYNC_asm

        jr win_loop

        dead_loop:
        
            call ds_DeadState
            call cpct_waitVSYNC_asm

        jr dead_loop

        

        init_game:
            call stopMusic
            ld a, #1
            ld (inGame), a
        game_loop:
            
            
            ld a, (menu_state_game)
            cp #0
            jr z, menu_init

            ;;== Check enemies spawning ==;;
            ;;call enemy_CreateNewEnemy

            ;;== Clears entities ==;;
            call game_Clear

            ld  a, (m_isPlayerDead)
            cp  #0
            jr  nz, not_update


            ;;== ==========ARROW FLICKER ANIMATION ==========;;
            ;;         Put #1 on arroFlickerActivation       ;;
            ;;===============================================;;

            call gui_ArrowFlicker

            ;;== Updates entities ==;;
            call game_Update

            not_update:
            
            ;;== Check for changes of game states==;;
            ld a, (menu_state_game)
            cp #0
            jr z, menu_init

            ld a, (winner_win_state)
            cp #0
            jr nz, win_loop

            ld a, (dead_state_game)
            cp #0
            jr nz, dead_loop

            ;;== Draws entities ==;;
            call game_Draw

            ;;== Wait Interruption ==;;
            call waitISR

            ;;== To synch with the raster to avoid FLICKERING ==;;
           	call cpct_waitVSYNC_asm

            ;;;== Switch the buffer to visualize the chnages of the game ==;;
            call buffer_switch

            ld  a, (m_isPlayerDead)
            cp  #0
            jp  z, game_loop

            ld  a, (m_isPlayerDead)
            dec a
            ld  (m_isPlayerDead), a
             

        jp    game_loop


;;=======================================================
;; game_CLEAR: Clears al the entities from the screen
;; DESTROYS: 
;;      XD -> All that you can imagine
;;========================================================

game_Clear:

    ;;===== Clear all entities ====== ;;
    ld  ix, #player_data ;;Player
    call ren_Clear
    ld  hl, #ren_Clear  ;;PowerUp
    call pu_DoForAll
    ld  hl, #ren_Clear ;;Bullet
    call bullet_DoForAll
    ld  hl, #ren_Clear  ;;EnemyBullet
    call bulletEn_DoForAll
    ld  hl, #ren_Clear ;;Enemy
    call enemy_DoForAll
    ld  hl, #ren_Clear   ;;Obstacles
    call obstacle_DoForAll


ret

;;=======================================================
;; game_UPDATE: Updates al the entities 
;; DESTROYS: 
;;      XD -> All that you can imagine
;;========================================================

game_Update:

    ;;===== Updates all the entities ==== ;;
   
    ld  hl, #ent_Update   ;;Obstacles
    call obstacle_DoForAll   
    
    ld  ix, #player_data  ;;Player
    call ent_Update
    
    ld  hl, #ent_Update   ;;PowerUp
    call pu_DoForAll
    ld  hl, #ent_Update   ;;Bullet
    call bullet_DoForAll
    ld  hl, #ent_Update   ;;Bullet
    call bulletEn_DoForAll
    ld  hl, #ent_Update   ;;Enemy
    call enemy_DoForAll
    call map_Update

ret

;;=======================================================
;; game_DRAW: Draw al the entities on the screen
;; DESTROYS: 
;;      XD -> All that you can imagine
;;========================================================

game_Draw:

    ;;===== Draw ===== ;;
    ld  a, (m_isPlayerDead)
    cp #0
    jr  nz, continue_drawing

    ld  ix, #player_data   ;;Player
    call ent_Draw

    continue_drawing:
    ld  hl, #ent_Draw      ;;PowerUp
    call pu_DoForAll
    ld  hl, #ent_Draw      ;;Bullet
    call bullet_DoForAll
    ld  hl, #ent_Draw   ;;Bullet
    call bulletEn_DoForAll
    ld  hl, #ent_Draw    ;;Enemy
    call enemy_DoForAll
    ld  hl, #ent_Draw      ;;Obstacles
    call obstacle_DoForAll
    

ret


game_SetPlayerDeadState:

    ld  a, #20
    ld  (m_isPlayerDead), a

ret 
