;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "main.h.s"
.include "player.h.s"
.include "entity.h.s"
.include "map.h.s"
.include "dobleBuffer.h.s"
.include "obstacle.h.s"
.include "powerUp.h.s"
.include "render.h.s"
.include "randomManager.h.s"
.include "enemy.h.s"
.include "collision.h.s"
.include "enemyBullet.h.s"
.include "deadScreen.h.s"
.include "packs.h.s"
.include "gui.h.s"

;;== SPAWN VARIABLES ==;;

scrollsCounter: .db #0
scrollsToPassRegion: .db #5
regionCounter: .db #10

m_next_element: .dw Level11
m_y_aux_pointer: .db #0
m_y_aux_pos: .db #0
m_x_aux_pos: .db #0
m_aux_rc: .db #0

DefineLevel11 Level11
DefineTransition1 Transition1
DefineLevel12 Level12
DefineTransition2 Transition2
DefineLevel21 Level21
DefineTransition3 Transition3
DefineLevel22 Level22
DefineTransition4 Transition4
DefineLevel31 Level31
DefineTransition5 Transition5
DefineLevel32 Level32
DefineTransition6 Transition6
DefineLevel41 Level41
DefineTransition7 Transition7

;;== STRING VARIABLES ==;;

string: .asciz "Level: "
currentLevel: .db #1
holeLevelsCompleted: .db #0
currentPhase: .db #1
scrolling: .db #1
                     
;;;================================
;;; MAP_UPDATE: Updates the current state
;;; DESTROY: HL, BC, DE, AF, IX
;;;================================

map_Update::

    ;;====== LEVEL COUNTER ======;;
    ;;call map_DrawLevelLabel
    ;;ld  hl, (currentLevel)
    ;;push hl                         
    ;;call _drawLevelLabel            
    ;;pop af'
    ;;===========================;;
    ld a, #0
    ld (scrolling), a

    ld ix, #player_data
    ld hl, #Key_W
    call key_isKeyPressed
    cp #1
    ret nz

    ld a, p_bm(ix)                  ;;Check obstacle down
    cp #4
    ret z

    ld a, de_y(ix)
    cp #100
    ret nz

    ld a, #1
    ld (scrolling), a

    ld  a, (scrollsCounter)         ;;A = Scroll Counter
    inc a                           ;;A++
    ;push af                         ;;| To save A
    ;ld  a, (scrollsToPassRegion)    
    ;ld  b, a                        ;;B = Number of scrolls to pass a region
    ;pop af                          ;;\
    ;cp b                            ;; ScrollCounter == ScrollsToPassRegion?
    cp #5
    jr  z, reset_sc

        ld  (scrollsCounter), a     ;;Update SCROLLS COUNTER
        jr  exit_spawning           ;;Exit the spawn logic

    ;;== Reset the Scrolls values ==;;
    reset_sc:
    ld  a, #0
    ld  (scrollsCounter), a
    ld  a, (regionCounter)
    inc a
    ld  (regionCounter), a

    ;;== Check if we have to spawn an Element at this point ==;;
    checK_other_spawn:          
    ld  b, a                        ;;B = REGION COUNTER
    ld  hl, (m_next_element)        ;;HL = DIR of the current Element in the queue
    ld  a, (hl)                     ;;A = Y position of THIS Element
    cp  b                           ;;If A < B -> Spawn THIS element
    jp  m, map_SpawnElement         ;;|

exit_spawning:   

    ;;Always move the screen entitie
    call map_MoveObstacles
    call map_MoveEnemies
    call map_MoveBullet
    call map_MovePU
    
ret

;;========================================================================================================;;
;;                                                                                                        ;;
;;                                         STATES METHODS                                                 ;;
;;                                                                                                        ;;
;;========================================================================================================;;

;;=========================================================
;;  map_SpawnElement:
;;=========================================================

map_SpawnElement:

    ld  hl, (m_next_element)            ;;HL = DIR of THIS Element
    inc hl                              ;;HL -> Element TYPE 
    ld  a, (hl)                         ;;A = TYPE
    
    
    ;;== ENEMY1 ==;;
    cp #0                               
    jr  nz, spawn_obs

        call map_SpawnEnemy1Element         ;;Spawns Enemy1 type of Element
        call map_ToNextElement              ;;M_NEXT_ELEMENT -> The next Element to evaluate

        ld  a, (regionCounter)              ;;A = REGION COUNTER
        jp  checK_other_spawn               ;;Go back to check if there's another Element in the same region  

    ;;== OBSTACLE1 ==;;

    spawn_obs:
    cp #2
    jr  nz, spawn_obs2

        call map_SpawnObstacle1Element      ;;Spawns Obsctale1 type of Element
        call map_ToNextElement              ;;M_NEXT_ELEMENT -> The next Element to evaluate

        ld  a, (regionCounter)              ;;A = REGION COUNTER
        jp  checK_other_spawn               ;;Go back to check if there's another Element in the same region


    ;;== OBSTACLE2 ==;;

    spawn_obs2:
    cp #1
    jr  nz, spawn_obs3

        call map_SpawnObstacle2Element      ;;Spawns Obsctale2 type of Element
        call map_ToNextElement              ;;M_NEXT_ELEMENT -> The next Element to evaluate

        ld  a, (regionCounter)              ;;A = REGION COUNTER
        jp  checK_other_spawn               ;;Go back to check if there's another Element in the same region

    ;;== OBSTACLE3 ==;;

    spawn_obs3:
    cp #3
    jr  nz, spawn_obs4

        call map_SpawnObstacle3Element      ;;Spawns Obsctale3 type of Element
        call map_ToNextElement              ;;M_NEXT_ELEMENT -> The next Element to evaluate

        ld  a, (regionCounter)              ;;A = REGION COUNTER
        jp  checK_other_spawn               ;;Go back to check if there's another Element in the same region

    ;;== LETAL OBSTACLE ==;;

    spawn_obs4:
    cp #4
    jr  nz, spawn_en2

        call map_SpawnObstacle4Element      ;;Spawns Obsctale4 (Letal) type of Element
        call map_ToNextElement              ;;M_NEXT_ELEMENT -> The next Element to evaluate

        ld  a, (regionCounter)              ;;A = REGION COUNTER
        jp  checK_other_spawn               ;;Go back to check if there's another Element in the same region

    ;;== ENEMY2 ==;;

    spawn_en2:
    cp #5                               
    jr  nz, spawn_pu

        call map_SpawnEnemy2Element         ;;Spawns Enemy2 type of Element
        call map_ToNextElement              ;;M_NEXT_ELEMENT -> The next Element to evaluate

        ld  a, (regionCounter)              ;;A = REGION COUNTER
        jp  checK_other_spawn               ;;Go back to check if there's another Element in the same region  

    ;;== POWER UP ==;;

    spawn_pu:
    cp #6                               
    jp  nz, percent_25

        call map_SpawnPUElement         ;;Spawns PowerUp type of Element
        call map_ToNextElement              ;;M_NEXT_ELEMENT -> The next Element to evaluate

        ld  a, (regionCounter)              ;;A = REGION COUNTER
        jp  checK_other_spawn               ;;Go back to check if there's another Element in the same region  

    
    percent_25:
    cp  #25
    jr  nz, percent_50

        call map_ToNextElement

        push hl
        ld  hl, #level_colours
        ld (final_colours), hl
        call gui_UpdateColours
        pop hl
        jp checK_other_spawn

    percent_50:
    cp  #50
    jr  nz, percent_75

        call map_ToNextElement

        push hl
        ld  hl, #level_colours50
        ld (final_colours), hl
        call gui_UpdateColours
        pop hl
        jp checK_other_spawn

    percent_75:
    cp  #75
    jr  nz, finalLevel11

        call map_ToNextElement

        push hl
        ld  hl, #level_colours75
        ld (final_colours), hl
        call gui_UpdateColours
        pop hl
        jp checK_other_spawn

    finalLevel11:
    cp #11
    jr  nz, trans1

        call map_ChangeLevel


    trans1:
    cp #51
    jr  nz, finalLevel12

        ;;Change the level label
        ld hl, #guiLevel2
        ld (guiLevelLabel), hl
        call gui_DrawLevelLabel

        call map_ChangeLevel
    
    finalLevel12:
    cp #12
    jr  nz, trans2

        push hl
        ld  hl, #level_colours100
        ld (final_colours), hl
        call gui_UpdateColours
        pop hl
         call map_ChangeLevel

    trans2:
    cp #52
    jr  nz, finalLevel21

        ;; === CHANGE SPRITES ==== ;;
        ld hl, #_level2_pack_end
        ld de, #descompress_buffer_end
        call cpct_zx7b_decrunch_s_asm

        ld hl, #guiLevel1
        ld (guiLevelLabel), hl

        call gui_DrawLevelLabel
        call gui_DrawColours
        call gui_DrawLevelPhoto

        ;;Add one life
        ld  ix, #player_data
        ld  a, p_lf(ix)
        cp  #3
        jr  z, next_step

            inc a
            ld  p_lf(ix), a
            call gui_DrawTopData

        next_step:
        
        ;; === ACTIVATE POWER UPS ==== ;;
        ld a , #1
        ld (level2), a

        call map_ChangeLevel
    
    finalLevel21:
    cp #21
    jr  nz, trans3

        call map_ChangeLevel
    
    trans3:
    cp #53
    jr  nz, finalLevel22

        ;;Change the level label
        ld hl, #guiLevel2
        ld (guiLevelLabel), hl
        call gui_DrawLevelLabel

        call map_ChangeLevel
    
    finalLevel22:
    cp #22
    jr  nz, trans4


        push hl
        ld  hl, #level_colours100
        ld (final_colours), hl
        call gui_UpdateColours
        pop hl
        call map_ChangeLevel

    trans4:
    cp #54
    jr  nz, finalLevel31

        ;; === CHANGE SPRITES ==== ;;
        ld hl, #_level3_pack_end
        ld de, #descompress_buffer_end
        call cpct_zx7b_decrunch_s_asm

        ld hl, #guiLevel1
        ld (guiLevelLabel), hl

        call gui_DrawLevelLabel
        call gui_DrawColours
        call gui_DrawLevelPhoto

        ;;Add one life
        ld  ix, #player_data
        ld  a, p_lf(ix)
        cp  #3
        jr  z, next_step2

            inc a
            ld  p_lf(ix), a
            call gui_DrawTopData
            
        next_step2:
        
        call map_ChangeLevel
    
    finalLevel31:
    cp #31
    jr  nz, trans5

        call map_ChangeLevel

    trans5:
    cp #55
    jr  nz, finalLevel32

        ;;Change the level label
        ld hl, #guiLevel2
        ld (guiLevelLabel), hl
        call gui_DrawLevelLabel

        call map_ChangeLevel

    finalLevel32:
    cp #32
    jr  nz, trans6

        push hl
        ld  hl, #level_colours100
        ld (final_colours), hl
        call gui_UpdateColours
        pop hl
        call map_ChangeLevel

    trans6:
    cp #56
    jr  nz, finalLevel41

        ;; === CHANGE SPRITES ==== ;;
        ld hl, #_level4_pack_end
        ld de, #descompress_buffer_end
        call cpct_zx7b_decrunch_s_asm

        ld hl, #guiLevel1
        ld (guiLevelLabel), hl

        call gui_DrawLevelLabel
        call gui_DrawColours
        call gui_DrawLevelPhoto

        ;;Add one life
        ld  ix, #player_data
        ld  a, p_lf(ix)
        cp  #3
        jr  z, next_step3

            inc a
            ld  p_lf(ix), a
            call gui_DrawTopData
            
        next_step3:
        ;; === ACTIVATE ENEMY3 ==== ;;
        ld a , #1
        ld (level3), a
        call map_ChangeLevel

    finalLevel41:
    cp #41
    jr  nz, trans7
        
        call map_ChangeLevel


    trans7:
    cp #57
    jr  nz, exit_spawn_logic

        ;;===== WINNER ======;;

        jp player_Win
        
    
exit_spawn_logic:
    jp exit_spawning
ret

;;=====================================================================
;;  map_ChangeLevel: Changes al the parameters needed to pass a Level
;;
;;  DESTROYS: 
;;        AF, HL
;;=====================================================================

map_ChangeLevel:

    ld hl, (m_next_element)         ;;HL = DIR of the last Element
    inc hl                          ;;Hl++ x3
    inc hl                          ;;|
    inc hl                          ;;|
    ld  (m_next_element), hl        ;;Now we point to the first elemento of the next level

    ld  a, #0                       ;;RegionCounter = 0
    ld  (regionCounter), a          ;;|

    ld  a, #0                       ;;ScrollsCounter = 0
    ld  (scrollsCounter), a         ;;|

ret

;;============================================================================
;;  map_SpawnObstacle1Element: 
;;
;;  DESTROYS:
;;         AF, BC, DE, HL, IY
;;============================================================================

map_SpawnObstacle1Element:

    ld  hl, #obstacle_data
    call obstacle_Spawn2                ;;Create the Obstcle
    ld  a, #0                           ;;A = resultant Y value
    ld  de_y(iy), a                     ;;DE_Y(IY) = A
    ld  hl, (m_next_element)            ;;HL = DIR of THIS Element
    inc hl                              
    inc hl                              ;;HL -> DIR of X position of the Element
    ld  a, (hl)                         ;;A = X position of the Element
    ld  de_x(iy), a                     ;;DE_X(IY) = A

ret

;;============================================================================
;;  map_SpawnObstacle2Element: 
;;
;;  DESTROYS:
;;         AF, BC, DE, HL, IY
;;============================================================================

map_SpawnObstacle2Element:

    ld  hl, #obstacle2_data
    call obstacle_Spawn2                ;;Create the Obstcle
    ld  a, #0                           ;;A = resultant Y value
    ld  de_y(iy), a                     ;;DE_Y(IY) = A
    ld  hl, (m_next_element)            ;;HL = DIR of THIS Element
    inc hl                              
    inc hl                              ;;HL -> DIR of X position of the Element
    ld  a, (hl)                         ;;A = X position of the Element
    ld  de_x(iy), a                     ;;DE_X(IY) = A

ret

;;============================================================================
;;  map_SpawnObstacle3Element: 
;;
;;  DESTROYS:
;;         AF, BC, DE, HL, IY
;;============================================================================

map_SpawnObstacle3Element:

    ld  hl, #obstacle3_data
    call obstacle_Spawn2                ;;Create the Obstcle
    ld  a, #0                           ;;A = resultant Y value
    ld  de_y(iy), a                     ;;DE_Y(IY) = A
    ld  hl, (m_next_element)            ;;HL = DIR of THIS Element
    inc hl                              
    inc hl                              ;;HL -> DIR of X position of the Element
    ld  a, (hl)                         ;;A = X position of the Element
    ld  de_x(iy), a                     ;;DE_X(IY) = A

ret

;;============================================================================
;;  map_SpawnObstacle4Element: 
;;
;;  DESTROYS:
;;         AF, BC, DE, HL, IY
;;============================================================================

map_SpawnObstacle4Element:

    ld  hl, #obstacle5_data
    call obstacle_Spawn2                ;;Create the Obstcle
    ld  a, #0                           ;;A = resultant Y value
    ld  de_y(iy), a                     ;;DE_Y(IY) = A
    ld  hl, (m_next_element)            ;;HL = DIR of THIS Element
    inc hl                              
    inc hl                              ;;HL -> DIR of X position of the Element
    ld  a, (hl)                         ;;A = X position of the Element
    ld  de_x(iy), a                     ;;DE_X(IY) = A

ret

;;============================================================================
;;  map_SpawnEnemy1Element: 
;;
;;  DESTROYS:
;;         AF, BC, DE, HL, IY
;;============================================================================

map_SpawnEnemy1Element:

    call enemy_Spawn2                   ;;Create the Enemy
    ld  a, #0            ;;A = resultant Y value
    ld  de_y(iy), a                     ;;DE_Y(IY) = A
    ld  hl, (m_next_element)            ;;HL = DIR of THIS Element
    inc hl                              
    inc hl                              ;;HL -> DIR of X position of the Element
    ld  a, (hl)                         ;;A = X position of the Element
    ld  de_x(iy), a                     ;;DE_X(IY) = A

ret

;;============================================================================
;;  map_SpawnEnemy2Element: 
;;
;;  DESTROYS:
;;         AF, BC, DE, HL, IY
;;============================================================================

map_SpawnEnemy2Element:

    call enemy2_Spawn2                   ;;Create the Enemy
    ld  a, #0            ;;A = resultant Y value
    ld  de_y(iy), a                     ;;DE_Y(IY) = A
    ld  hl, (m_next_element)            ;;HL = DIR of THIS Element
    inc hl                              
    inc hl                              ;;HL -> DIR of X position of the Element
    ld  a, (hl)                         ;;A = X position of the Element
    ld  de_x(iy), a                     ;;DE_X(IY) = A

ret

;;============================================================================
;;  map_SpawnPUElement: 
;;
;;  DESTROYS:
;;         AF, BC, DE, HL, IY
;;============================================================================

map_SpawnPUElement:

    call pu_Spawn                   ;;Create the Enemy
    ld  a, #0            ;;A = resultant Y value
    ld  de_y(iy), a                     ;;DE_Y(IY) = A
    ld  hl, (m_next_element)            ;;HL = DIR of THIS Element
    inc hl                              
    inc hl                              ;;HL -> DIR of X position of the Element
    ld  a, (hl)                         ;;A = X position of the Element
    ld  de_x(iy), a                     ;;DE_X(IY) = A

ret

;;============================================================================
;;  map_TONEXTELEMENT: 
;;
;;  DESTROYS:
;;          HL, DE
;;============================================================================

map_ToNextElement:

    ld  hl, (m_next_element)
    ld  de, #3
    add hl, de
    ld  (m_next_element), hl 

ret

;;========================================================================================================;;
;;                                                                                                        ;;
;;                                         MOVEMENT METHODS                                               ;;
;;                                                                                                        ;;
;;========================================================================================================;;

;;==============================================================================
;; MAP_MOVEOBSTACLES: Move obstacles with the scroll and check the collision 
;; DESTROY: AF
;;==============================================================================

map_MoveObstacles:
    call obstacle_GetNumObstacle 
    ld a, (bc)
    cp #0
    jr z, dont_moveObstacles
        ld hl, #obstacle_MoveScroll     ;;Add the function that we want to call
        call obstacle_DoForAll
    dont_moveObstacles:
ret

;;============================================================================
;; MAP_MOVEENEMIES: Move obstacles with the scroll and check the collision 
;; DESTROY: AF
;;============================================================================

map_MoveEnemies:

    call enemy_GetEnemySize
    ld a, (bc)
    cp #0
    jr z, dont_moveEnemies

        ld hl, #enemy_MoveScroll     ;;Add the function that we want to call
        call enemy_DoForAll

    dont_moveEnemies:
ret

;;============================================================================
;; MAP_MOVEBULLET: Move obstacles with the scroll and check the collision 
;; DESTROY: AF
;;============================================================================

map_MoveBullet:

    call bulletEn_GetBulletSize
    ld a, (bc)
    cp #0
    jr z, dont_moveBullets

        ld hl, #bulletEn_MoveScroll     ;;Add the function that we want to call
        call bulletEn_DoForAll

    dont_moveBullets:
ret

;;============================================================================
;; MAP_MOVEENEMIES: Move obstacles with the scroll and check the collision 
;; DESTROYS: 
;;      AF
;;============================================================================

map_MovePU:

    call pu_GetPUSize
    ld a, (bc)
    cp #0
    jr z, dont_movePU

        ld hl, #pu_MoveScroll       ;;Add the function that we want to call
        call pu_DoForAll

    dont_movePU:
ret


