;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "bullet.h.s"
.include "main.h.s"
.include "enemy.h.s"
.include "obstacle.h.s"
.include "randomManager.h.s"
.include "player.h.s"
.include "render.h.s"
.include "entity.h.s"
.include "collision.h.s"
.include "packs.h.s"

;;LOCAL VARIABLES
m_next_obstacle: .dw obstacle_vector       ;;Pointer of the memory position of the next entity 
m_num_obstacle: .db 00                   ;;Counter to see how many entities we have created
obstacle_mem: .dw 0x0000
;m_last_obstacle: .dw 0x0000

;;============================== CONSTANTS =======================================;;

;;== Obstacles ==;;
k_max_num_obstacles = 10                  ;;Constant for the maximum number of obstacles
k_obstacle_size = obstacle_vector0_size     ;;Dynamic size of obstacle

;;== Obstacle vector ==;;
obstacle_vector:
DefineNObstacles obstacle_vector, k_max_num_obstacles 

m_coll_acc: .db #0

;;Animation
m_typeAnimation:  .db #0



;;== Default obstacle for new instances, i.e. the default constructor ==;;
DefineObstacle obstacle_data, 0x20, 0x00, 0x00, 0x00, 0x20, 0x20, obstacleBig, 0x00, 0x00, 0x00, 0x00, obstacle_Update, 0, 0, -1, -1
DefineObstacle obstacle2_data, 0x10, 0x00, 0x00, 0x00, 0x10, 0x20, obstacleMiddle, 0x00, 0x00, 0x00, 0x00, obstacle_Update, 0, 0, -1, -1
DefineObstacle obstacle3_data, 0x15, 0x00, 0x00, 0x00, 0x08, 0x20, obstacleSmall, 0x00, 0x00, 0x00, 0x00, obstacle_Update, 0, 0, -1, -1
DefineObstacle obstacle4_data, 0x15, 0x00, 0x00, 0x00, 0x20, 0x20, obstacleBig, 0x00, 0x00, 0x00, 0x00, obstacle_Update, 0, 0, -1, -1
DefineObstacle obstacle5_data, 0x15, 0x00, 0x02, 0x00, 0x10, 0x20, _assassinObstacle_0, 0x00, 0x00, 0x00, 0x00, obstacle_Side2Side, 0, 1, 0, 0

;;==============================================================================
;; obstacle_New: Register a new obstacle
;; INPUT: 
;; DESTROYS: AF, BC, HL 
;; OUTPUT: HL -> Pointer to the current entity that we are going to create
;;==============================================================================

obstacle_New:
    ;;
    ;;We add one more entity at the counter.
    ;;
    push hl                        ;;HL = Pointer to obstacle data
    ld a, (m_num_obstacle)               
    inc a
    ld (m_num_obstacle), a

    ;;
    ;;We add the size of the entity to get the position of the next entity 
    ;;and we return the pointer to the current entity in HL
    ;;
    ld hl, (m_next_obstacle)          
    ld bc, #k_obstacle_size
    add hl, bc
    ld (m_next_obstacle), hl
    or  a                          ;;Or of a to destroy the carry
    sbc hl, bc                     ;;Subtract HL - BC to get the pointer to the current entity

    ex de, hl
    pop hl
ret

;;============================================================
;; obstacle_COPY: Copy the values of one obstacle to another 
;; DESTROY: IX, IY
;; INPUT:   HL -> Origin obstacle
;;          DE -> Destinatio obstacle
;;          BC -> Size of the copy
;;============================================================

obstacle_CopyDefault:

    ld (obstacle_mem), de
    ld iy, (obstacle_mem)    
    ld  bc, #k_obstacle_size         ;;Copy the size of the entity
    ldir

ret

;;============================================================
;; obstacle_COPY: Copy the values of one obstacle to another 
;; DESTROY: IX, IY
;; INPUT:   HL -> Origin obstacle
;;          DE -> Destinatio obstacle
;;          BC -> Size of the copy
;;============================================================

obstacle_Copy:

    ld (obstacle_mem), de
    ld iy, (obstacle_mem)    
    ld  bc, #k_obstacle_size         ;;Copy the size of the entity
    ldir

    call collision_GetSpawningX
    ld  de_x(iy), a

ret

;;============================================================
;;
;;============================================================

obstacle_Spawn:

    ld  a,  #k_max_num_obstacles
    push af
    ld  a, (m_num_obstacle)
    ld  b, a
    inc b
    pop  af
    cp  b
    ret z

    call obstacle_New
    call obstacle_Copy

ret

;;============================================================
;;
;;============================================================

obstacle_Spawn2:

    ld  a,  #k_max_num_obstacles
    push af
    ld  a, (m_num_obstacle)
    ld  b, a
    inc b
    pop  af
    cp  b
    ret z

    call obstacle_New
    call obstacle_CopyDefault

ret
;;===============================================================
;; obstacle_DoForAll: Update, draw and clear to all entities
;; INPUT:  HL-> Function 
;;===============================================================

obstacle_DoForAll:

    ld a, (m_num_obstacle)         ;;Loads the number of the entities 
    cp #0
    jr z, not_ob 

    ld ix, #obstacle_vector        ;;We load the first entity of the vector in IX
    ld (method), hl                ;;Put the function in method
    
    loop:
        push af                    ;;Push a on the stack to not lose them

        ld  a, o_dead(ix)
        cp  #1
        jr  z, dead

        method = . + 1             ;;Current position + 1 = the function of the next call
        call obstacle_DoForAll     ;;We call the method we have in hl 

        dead:
        pop af

        ;;
        ;;We put ix pointing to the next entity, 
        ;;so that the next time I draw it with that entity
        ;;
        ld bc, #k_obstacle_size
        add ix, bc
        dec a 
    jr nz, loop

not_ob:
ret

;;=======================================================================
;; obstacle_EveryObstacle: Iterate through all the obstacles
;; INPUT:
;;    IX -> HERO
;;    IY -> Pointer to the obstacles vector
;;    AF -> Number of active obstacles
;;    BC -> Size of an enemy  
;;        
;;=======================================================================

obstacle_EveryObstacle:

    ld a, (m_num_obstacle)              ;;A = Number 
    cp #0
    jr z, no_obst
 
    ld iy, #obstacle_vector             ;;IY = Pointer to the enemies vector            
    ld (met), hl

    loop_dfe:
        push af                         ;Push a on the stack to not lose it

        ld  a, o_dead(iy)
        cp  #1
        jr  z, dead2
        met = . + 1                     ;;Current position + 1 = the function of the next call
        call obstacle_Collisions

        dead2:
        pop af
        ;;
        ;;We put ix pointing to the next entity, 
        ;;so that the next time I draw it with that entity
        ;;
        ld bc, #k_obstacle_size         ;;Move M_NEXT_obstacle pointer to the next position by the obstacles size
        add iy, bc                 
        dec a                           ;;Next iteration
    jr nz, loop_dfe

no_obst:
ret

;;============================================================
;; obstacle_Update: Do nothing
;;============================================================

obstacle_Update:
ret

;;============================================================
;; obstacle_Side2Side: The letal obstacle move side to side of the screen
;;============================================================

obstacle_Side2Side:
    ;;
    ;;Walk from left to right
    ;;
    walk:
        call obstacle_Animation

        ;;check right wall
        ld a, de_x(ix)
        cp #80-16
        jr z, move_to_left
        jp p, move_to_left

        
        ;;check left wall
        ld a, de_x(ix)
        cp #16
        jr z, move_to_right
        jp m, move_to_right

        jp ent_Move

        ;;change direction on the left
        move_to_left:
        ld o_vx(ix), #-2
        jp ent_Move

        ;;change direction on the right
        move_to_right:
        ld o_vx(ix), #2
        jp ent_Move

ret

;;=======================================================================
;; obstacle_Animation: Changes the obstacle's sprite
;; INPUT:
;;    IX -> Obstacle
;;        
;;=======================================================================
obstacle_Animation:
    
    ;;
    ;;Check the clock and if it's 2 changes the sprite
    ;;
    ld a, o_clockAnimation(ix)
    cp #2
    jr nz, animationRight
        ld a, o_typeAnimation(ix)
        cp #1
        jr z, secondMovement
        ;;
        ;;First frame
        ;;
            ld hl, #_assassinObstacle_0                    ;;Changes the sprite
            ld de_sprite_l(ix), l
            ld de_sprite_h(ix), h
            ld a, #1
            ld o_typeAnimation(ix), a                      ;;Set the next frame (2nd frame)

            ld a, #0                                       ;;Reset the clock
            ld o_clockAnimation(ix), a
        ret

        ;;
        ;;Second frame
        ;;
        secondMovement:
            ld hl, #_assassinObstacle_1
            ld de_sprite_l(ix), l
            ld de_sprite_h(ix), h
            ld a, #0

            ld o_typeAnimation(ix), a                       ;;Set the next frame (1nd frame)
            ld o_clockAnimation(ix), a                      ;;Reset the clock
        ret
    animationRight: 

    inc a                                                   ;;Inc clock
    ld o_clockAnimation(ix), a   
ret

;;=======================================================
;; OBSTACLE_COLLISION: Check the collision 
;; INPUT: IX -> HERO, ENEMY
;;        IY -> OBSTACLE
;;=======================================================

obstacle_Collisions:
    ;;
    ;;Save if this obstacle kill
    ;;
    
    ld a, o_kill(iy)
    
    push ix
    ld ix, #player_data
    ld p_kill(ix), a
    pop ix
    

    call collision_CheckEntitiesCollision

    cp #0                ;;NO COLLISION
    jr z, n_col

    ;;COLLISION

    ld a, o_kill(iy)
    cp #1
    jr nz, jump

    ld a, o_vx(iy)
    cp #2
    jr z, set_minus_two
        ld a, #2
        ld o_vx(iy),a
        jr jump
        
    set_minus_two:
        ld a, #-2 
        ld o_vx(iy), a
    
    jump:

    call obstacle_CollisionsDirection
    pop af 
    pop af
    ret

    n_col:
    ld a, #0
    ld p_bm(ix), a
ret

;;=============================================================
;; obstacle_BulletColissions: Check the collision between 
;; the array of obstacle and the array of bullets
;; INPUT: IX -> BULLET
;;        IY -> OBSTACLE
;;=============================================================

obstacle_BulletCollisions:
    
    call collision_CheckEntitiesCollision

    cp #0                ;;NO COLLISION
    jr z, no_collision

    ;;COLLISION

    ld b_lft(ix), #0
    
    no_collision:
ret

;;=============================================================
;; obstacle_BoundingBoxColissions: Check the visiblity
;; INPUT: IX -> BULLET
;;        IY -> OBSTACLE
;;=============================================================

obstacle_BoundingBoxCollisions:
    
    call collision_CheckEntitiesCollision

    cp #0                ;;NO COLLISION
    jr z, no_collisionBB

    ;;COLLISION

    pop af 
    pop af
    ld a, #5
    
    no_collisionBB:
ret

;;========================================================================
;; OBSTACLE_COLLISIONDirection: Check the direction of the collision 
;; INPUT: IX -> HERO
;;        IY -> OBSTACLE
;; DESTROY: A, BC, D, HL
;; OUTPUT: A -> movement_block
;;========================================================================

obstacle_CollisionsDirection:
    direction_x:
        ld a, de_x(ix)
        ld c, de_x(iy)
        cp c

        jr z, block_movement_right
        jp m, block_movement_right

        block_movement_left:
            ld a, de_x(iy)
            ld b, de_w(iy)
            ld c, de_x(ix)
            add b           
            sub c 

            ld d, a
            ld h, #-1

            jp direction_y

        block_movement_right:
            ld a, de_x(ix)
            ld b, de_w(ix) 
            ld c, de_x(iy)
            add b       
            sub c 

            ld d, a
            ld h, #1
    
    direction_y:
        ld a, de_y(ix)
        ld c, de_y(iy)
        cp c

        jr z, block_movement_down
        jp m, block_movement_down

        block_movement_up:
            ld a, de_y(iy)
            ld b, de_h(iy)
            ld c, de_y(ix)
            add b           
            sub c 

            ld l, #-1

            jp decide_direction

        block_movement_down:
            ld a, de_y(ix)
            ld b, de_h(ix) 
            ld c, de_y(iy)
            add b       
            sub c 

            ld l, #1

    decide_direction:
        cp d

        jr z, block_movement_y
        jp m, block_movement_y

        block_movement_x:
            ld a, h
            cp #-1
            jr z, bl
                
            br:
                ld a, #1
                jr exit
            bl:
                ld a, #2
                jr exit

        block_movement_y:
            ld a, l
            cp #-1
            jr z, bu
            bd:
                ld a, #3
                jr exit
            bu:
                ld a, #4
                jr exit
    exit:
    ld en_bm(ix), a
ret

;;===========================================================================
;; obstacle_DestroyEveryObstacle: destroy every obstacle (captain obvious)
;; DESTROY: HL, A
;;===========================================================================

obstacle_DestroyEveryObstacle:

    push ix
    ld hl, #obstacle_Destroy
    call obstacle_DoForAll
    pop ix

    ld a, #00
    ld (m_num_obstacle), a        ;;Set number of obstacle = 0
    ld hl, #obstacle_vector      
    ld (m_next_obstacle), hl      ;;Set next entity to the array's first position

ret

;;============================================================
;; obstacle_DESTROY: Deletes a bullet from the memory
;; INPUT:   
;;    IX -> Pointer to THIS bullet
;; DESTROYS:
;;    AF, HL, DE, BC
;;============================================================

obstacle_Destroy:
    call ren_Clear_FrontBack
    
    ld  a, (m_num_obstacle)
    cp  #0
    jr  z, exit_destroy

        call obstacle_TurnBackNextObstacle   ;;M_NEXT_obstacle one position back
        call obstacle_DecreaseNumObstacles   ;;M_NUM_obstacle--
        
        push ix
        pop de                        

        ld  a, h                        ;;Check if the last obstacle is the same as THIS
        cp  d                           ;;|
        jr nz, make_copy                ;;|

        ld  a, l                        ;;|
        cp  e                           ;;|
        jr nz, make_copy                ;;|

        call obstacle_SetDeadState      ;;In other case, don't make a Copy,
        ret	                            ;;it's the same direction
                                        ;;only set this position (the last) to dead
        make_copy:

        ld  bc, #k_obstacle_size        ;;\Copy the last obstacle of the vector
        ldir                            ;;\to the position of THIS obstacle that we want to destroy
        call obstacle_SetDeadState                 
    
        
exit_destroy:
ret

;;=======================================================================
;; obstacle_TURNBACKNEXTBULLET: Set back next bullet pointer
;; DETROY: 
;;      AF, HL, BC 
;;=======================================================================

obstacle_TurnBackNextObstacle:
    push ix
    ld  hl, (m_next_obstacle)   ;;HL = Pointer to the last active obstacle 
    ld  bc, #k_obstacle_size                  
    or a 
    sbc hl, bc

    push hl
    pop ix
    ld  a, de_y(ix)
    add #4
    ld  de_y(ix), a
    
    ld  (m_next_obstacle), hl  
    pop ix
ret

;;=======================================================================
;; obstacle_DECREASENUMobstacleS: Decrease the number of obstacles 
;; DETROY: 
;;      AF 
;;=======================================================================

obstacle_DecreaseNumObstacles:

    ld  a, (m_num_obstacle)    ;;A = number of active obstacles
    dec a                      ;;A --
    ld  (m_num_obstacle), a    ;;<>
ret

;;=======================================================================
;; obstacle_SETDEADSTATE: Set the last obstacle's dead state to 1
;; DETROY: 
;;      AF
;;=======================================================================

obstacle_SetDeadState:
    push ix

        ld ix, (m_next_obstacle)
        ld a, #1
        ld o_dead(ix), a
        
    pop ix 
ret

;;=============================================================
;; obstacle_GETNUMOBSTACLES: Get obstacle number
;; OUTPUT: bc -> m_num_obstacle
;;=============================================================

obstacle_GetNumObstacle:
    ld bc, #m_num_obstacle
ret

;;==============================================================
;; obstacle_MOVESCROLL: Move obstacles with scroll and kill it 
;; OUTPUT: bc -> m_num_obstacle
;;==============================================================

obstacle_MoveScroll:

    ;;
    ;;Check if this is the end
    ;;
    ld a, de_y(ix)
    ld b, a
    ld a, de_h(ix)
    add b
    cp #200-2
    jp m, dont_moveObstacle
    jr c, dont_moveObstacle  

        jp obstacle_Destroy

    dont_moveObstacle:
        ld a, de_y(ix)
        add #4
        ld de_y(ix), a
        ret

;;=============================================================
;; obstacle_SETPOSITION: Set position of obstacles
;; OUTPUT: bc -> m_num_obstacle
;;=============================================================

obstacle_setPosition:
    ld ix, #obstacle_vector
    ld de_y(ix), #0
ret




obstacle_sts:

    ld  a, #0
    ld  (m_coll_acc), a

    ld  hl, #collision_CheckEntitiesCollision
    call obstacle_EveryObstacle

   ld  a, (m_coll_acc)
   cp  #1
   jr  z, moveit

       ;;ld  a, o_vx(ix)
       ;;cp  #4
       ;;jr  z, move_right
;;
       ;;    ld  o_vx(ix), #-4
       ;;    jp  ent_Move
;;
       ;;move_right:
       ;;ld  o_vx(ix), #4

       ld   a, #0xF0
       ld   hl, #0xC000
       ld   (hl), a
       
       ld   a, #0xF0
       ld   hl, #0x8000
       ld   (hl), a

   moveit:
   jp ent_Move

ret

obstacle_CheckOtherOBSCollision:
    ;;
    ;;Check collision in X axis
    ;;
    call check_entities_collision_x

    ;;
    ;;Make if statement
    ;;
    jr z, no_collisionOO
    jp m, no_collisionOO

    ;;
    ;;If i collide in x, then i check y
    ;;
    call check_entities_collision_y
    
    jr z, no_collisionOO
    jp m, no_collisionOO

    
    ;;
    ;;If there is collision
    ;;
    ld  a, (m_coll_acc)
    inc a
    ld  (m_coll_acc), a
    ret
    
    no_collisionOO:
    ld  a, #0

ret