;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "player.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "render.h.s"
.include "bullet.h.s"
.include "obstacle.h.s"
.include "enemy.h.s"
.include "menu.h.s"
.include "deadScreen.h.s"
.include "powerUp.h.s"
.include "dobleBuffer.h.s"
.include "map.h.s"
.include "enemyBullet.h.s"
.include "gui.h.s"
.include "winner.h.s"
.include "sound.h.s"


;; == Player local variables == ;;
m_cycles: .db 0x00
m_fire_delay: .db 0x01
m_has_laser: .db 0x00
m_playerAnimation: .db #0
m_isFiring: .db #0
m_isMoving: .db #0

;; == Player is not moving == ;;
spawn_enemies: .db 0
moved: .db 0
time_no_moved: .db 40
default_timer: .db 40

bullet_aux_x: .db #0
bullet_aux_y: .db #-8

animationClock: .db #0

movex: .db #0



;; == Aux Variables == ;;
pos_mem: .dw 0x0000  

;; == Level With Type 3 Enemies == ;;
level3: .db 0

;; == Player DEFINES == ;;
DefinePlayer player_data, 0x28, 0x90, 0x01, 0x01, 0x04, 0x10, _playerUp_0, 0x00, 0x00, 0x00, 0x00, player_CheckKeyboard, 0, 0, 0, 3
DefinePlayer player_backup, 0x28, 0x90, 0x01, 0x01, 0x04, 0x10, _playerUp_0, 0x00, 0x00, 0x00, 0x00, player_CheckKeyboard, 0, 0, 0, 3

;;========================================================================================================;;
;;                                                                                                        ;;
;;                                         PLAYER PRIVATE METHODS                                         ;;
;;                                                                                                        ;;
;;========================================================================================================;;

;;===============================================================================
;; PLAYER_CHECKKEYBOARD: Check the keyboard inputs and moves or fires a bullet
;; INPUTS: 
;;     IX -> Pointer to player
;; DESTROYS:
;;     AF, HL, BC
;;================================================================================

player_CheckKeyboard:

    call player_CheckMovementKeyboard
    call player_CheckFireKeyboard

ret


player_CheckMovementKeyboard:

    ld  a, p_lf(ix)
    ld hl, #obstacle_Collisions
    call obstacle_EveryObstacle
    call player_CheckLetalObstacle

    ld p_vx(ix), #0x00                     ;;Hero X Velocity = 0  
    ld p_vy(ix), #0x00                     ;;Hero Y Velocity = 0
   
    check_desp_x:

        ld hl, #Key_D
        call key_isKeyPressed           ;;Checks a specific Key by the given Key_Code
        cp  #0                          ;;A != 0? -> Key pressed
        jr  nz, d_pressed               ;;|

        ld hl, #Key_A
        call key_isKeyPressed           ;;Checks a specific Key by the given Key_Code
        cp  #0                          ;;A != 0? -> Key pressed
        jr  nz, a_pressed               ;;|
        jp  check_desp_y                ;;Jump to check_desp_y to avoid d_pressed y case false

        d_pressed:
            ld  b, #0
            ld  c, #1
            call player_CheckMovement
            jr check_desp_y

        a_pressed:
            ld  b, #0
            ld  c, #-1
            call player_CheckMovement

   check_desp_y:

        ld hl, #Key_W
        call key_isKeyPressed           ;;Checks a specific Key by the given Key_Code
        cp  #0                          ;;A != 0? -> Key pressed
        jr  nz, w_pressed               ;;|

        ld hl, #Key_S
        call key_isKeyPressed           ;;Checks a specific Key by the given Key_Code
        cp  #0                          ;;A != 0? -> Key pressed
        jr  nz, s_pressed               ;;|
        jp  exit_player_movement           

        w_pressed:
            ld  b, #1
            ld  c, #1
            call player_CheckMovement
            jr exit_player_movement

        s_pressed:
            ld  b, #1
            ld  c, #-1
            call player_CheckMovement


    exit_player_movement:
    ld  a, (m_isFiring)
    cp  #1
    jr  z, only_move

    call player_changeSprite

    only_move:
    call player_CheckMoveUp
    call ent_Move

ret


player_CheckFireKeyboard:

        ld a, (m_fire_delay)
        dec a
        jr  nz, update_delay

        ld  a, #0
        ld  (bullet_aux_x), a
        ld  a, #0
        ld  (bullet_aux_y), a

        ld hl, #Key_J
        call key_isKeyPressed           ;;Checks a specific Key by the given Key_Code
        cp  #0                          ;;A != 0? -> Key pressed
        jr  nz, j_pressed               ;;|

        ld hl, #Key_L
        call key_isKeyPressed           ;;Checks a specific Key by the given Key_Code
        cp  #0                          ;;A != 0? -> Key pressed
        jr  nz, l_pressed               ;;|
        jp  check_fire_y               ;;Jump to check_desp_y to avoid d_pressed y case false

        j_pressed:
        ld  a, #-2
        ld  (bullet_aux_x), a
        jr check_fire_y

        l_pressed:
        ld  a, #2
        ld  (bullet_aux_x), a

    check_fire_y:

        ld hl, #Key_I
        call key_isKeyPressed           ;;Checks a specific Key by the given Key_Code
        cp  #0                          ;;A != 0? -> Key pressed
        jr  nz, i_pressed               ;;|

        ld hl, #Key_K
        call key_isKeyPressed           ;;Checks a specific Key by the given Key_Code
        cp  #0                          ;;A != 0? -> Key pressed
        jr  nz, k_pressed               ;;|
        jp  exit_keyborad_fire_logic

        i_pressed:
        ld  a, #-8
        ld  (bullet_aux_y), a
        jr  exit_keyborad_fire_logic

        k_pressed:
        ld  a, #8
        ld  (bullet_aux_y), a

    exit_keyborad_fire_logic:

        ld  a, (bullet_aux_x)
        cp #0
        jr  nz, wants_bullet

        ld  a, (bullet_aux_y)
        cp #0
        jr  nz, wants_bullet

            ld  a, #0
            ld  (m_isFiring), a            
            jr  exit_keyborad_fire_method
            
        wants_bullet:
        ld a, #4
        ld  (m_fire_delay), a
        ld  a, #1
        ld  (m_isFiring), a
        call player_FireBullet
        call player_changeSpriteByFiring
        jr  exit_keyborad_fire_method

update_delay:
ld  (m_fire_delay), a

exit_keyborad_fire_method:
ret



;;=========================================================================
;; KEY_ISKEYPRESSED: Uses CPCtelera routines 
;; to check if a given Key is pressed
;; INPUTS:
;;      HL -> Specific Key_Code to check
;; OUTPUTS:
;;       A -> Pressed/Not pressed status (1 == pressed, 0 == not pressed)
;;=========================================================================

key_isKeyPressed:

    push hl
    call cpct_scanKeyboard_asm      ;;Checks the status of the Keyboard
    pop hl
    call cpct_isKeyPressed_asm      ;;Checks a specific Key by the given Key_Code
    jr nz, key_pressed

        ld  a, #0                   ;;Key not pressed
        ret
    
    key_pressed:
    ld  a, #1                       ;;Key pressed

ret

;;============================================================================
;; hero_CheckMovement: Checks if a given movement is posible and change the
;; INPUTS:
;;      BC => Needed movement (B = axis {x -> 0, y -> 1}, C = direction)
;;      IX => Puntero a entidad
;;============================================================================

player_CheckMovement:

   ld  a, b
   cp  #0
   jr  nz, check_mov_y        

       ;;
       ;;Check the X axis direction
       ;;

       ld  a, c                   
       cp  #1
       jr  nz, check_left_dir

           ld a, de_x(ix)           ;;Check RIGHT side of screen
           cp #80-4-1
           jr z, dont_move_x       ;;If we can't move, set E_VX = 0
          
           ld a, p_bm(ix)          ;;Check obstacle right
           cp #1
           jr z, killx

           ld p_vx(ix), #1
           jr chk_finished
                  
       check_left_dir:

           ld a, de_x(ix)           ;;Check LEFT side of screen
           cp #0+16
           jr z, dont_move_x       ;;If we can't move, set E_VX = 0
           
           ld a, p_bm(ix)          ;;Check obstacle left
           cp #2
           jr z, killx

           ld p_vx(ix), #-1
           jr chk_finished

        killx:
            ;;Check if the obstacle can kill
            ld a, p_kill(ix)
            cp #1
            jr nz, dont_move_x
            
                call player_HitByObstacle

        dont_move_x:
            ld p_vx(ix), #0
            jr chk_finished

   check_mov_y:

       ;;
       ;;Check the Y axis direction
       ;;
      
       ld  a, c                   
       cp  #-1
       jr  nz, check_down_dir

           ld a, de_y(ix)           ;;Check UP side of screen
           cp #200-16-4
           jr z, dont_move_y       ;;If we can't move, set E_VY = 0
           
           ld a, p_bm(ix)          ;;Check obstacle up
           cp #3
           jr z, killy

           ld p_vy(ix), #4
           jr chk_finished
                  
       check_down_dir:
           ld a, p_bm(ix)          ;;Check obstacle down
           cp #4
           jr z, killy

           ld a, de_y(ix)          ;;Check DOWN side of screen
           cp #100
           jr z, dont_move_y          ;;If we are in the middle of the screen, we move the map

           ld p_vy(ix), #-4
           jr chk_finished


        killy:
            ;;Check if the obstacle can kill
            ld a, p_kill(ix)
            cp #1
            jr nz, dont_move_y
            
                call player_HitByObstacle

        dont_move_y:
            ld p_vy(ix), #0
    

chk_finished: 

ret

player_CheckLetalObstacle:

           ld a, p_bm(ix)          ;;Check obstacle right
           cp #1
           jr z, kill

           ld a, p_bm(ix)          ;;Check obstacle left
           cp #2
           jr z, kill

        ret

        kill:
            ;;Check if the obstacle can kill
            ld a, p_kill(ix)
            cp #1
            jr nz, do_not_kill
            
                call player_HitByObstacle

    do_not_kill:

ret

;;============================================================================
;; player_CheckMoveUp: Checks if player is scrolling or not to spawn type 3 enemies
;; 
;;============================================================================

player_CheckMoveUp:

    ld a, (moved)
    cp #1
    jr z, was_moving

    not_moving:                ;; last cycle player -> not moving
        ld a, (scrolling)
        cp #0
        jr z, still_not_moving

        already_moving:         ;; now player -> moving
            ld a, #1
            ld (moved), a

            ld a, (default_timer)
            ld (time_no_moved), a   ;; reset counter 

            ld a, #0
            ld (spawn_enemies), a    ;;not spawn_enemies

            ret

        still_not_moving:          ;; now player -> not moving
            ld a, (time_no_moved)
            dec a                       ;;decrease counter
            cp #3
            jp p, not_up_arrow

            ld a, #1
            ld (arrowFlickerActivation), a
            not_up_arrow:

            cp #0                       ;; if counter == 0 -> spawn enemies
            ld (time_no_moved), a

            jr z, spawn_enemies_down   
            ret

            spawn_enemies_down:         ;;spawn enemies
            ld a, #1
            ld (time_no_moved), a

            ld a, (level3)               ;;if its not level 3 or more we cant spawn enemies
            cp #0
            jr z, cant_spawn_yet

            ld (spawn_enemies), a

            cant_spawn_yet:
            ret


    was_moving:                 ;; last cycle player -> moving
        ld a, (scrolling)
        cp #0
        jr z, stopped
        ret

        stopped:                ;; now player -> not moving
        ld a, #0
        ld (moved), a           

    ;;Change the sprite
    
ret


player_changeSprite:

    ;;
    ;;Right
    ;;
    sprite12:
        ld a, p_vx(ix)
        cp #1
        jr nz, sprite22
        ld a, p_vy(ix)
        cp #0
        jr nz, sprite22

        ;;======Changes frame==========;;
            ;;
            ;;Checks clock
            ;;
            ld a, (animationClock)
            cp #2
            jr nz, dontChangeSpriteRight
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameRight                     ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerRight_0
                ld a, #1
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2
            ;;
            ;;Frame 2
            ;;
            playerFrameRight:
                ld hl, #_playerRight_1
                ld a, #0
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2

            dontChangeSpriteRight:

            inc a
            ld (animationClock), a
        ;;============================;;

    ;;
    ;;Left
    ;;
    sprite22:
        ld a, p_vx(ix)
        cp #-1
        jr nz, sprite32
        ld a, p_vy(ix)
        cp #0
        jr nz, sprite32

        ;;======Changes frame==========;;
            ;;
            ;;Checks clock
            ;;
            ld a, (animationClock)
            cp #2
            jr nz, dontChangeSpriteLeft
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameLeft                    ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerLeft_0
                ld a, #1
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2
            ;;
            ;;Frame 2
            ;;
            playerFrameLeft:
                ld hl, #_playerLeft_1
                ld a, #0
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2

            dontChangeSpriteLeft:

            inc a
            ld (animationClock), a
        ;;============================;;

    ;;
    ;;Up
    ;;
    sprite32:
        
        ld a, p_vx(ix)
        cp #0
        jr nz, sprite42
        
        ld a, (scrolling)
        cp #1
        jr z, changeSpriteScrollingUp
            ld a, p_vy(ix)
            cp #-4
            jr nz, sprite42
        changeSpriteScrollingUp:
        ;;======Changes frame==========;;
            ;;
            ;;Checks clock
            ;;
            ld a, (animationClock)
            cp #2
            jr nz, dontChangeSpriteUp
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameUp                    ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerUp_0
                ld a, #1
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2
            ;;
            ;;Frame 2
            ;;
            playerFrameUp:
                ld hl, #_playerUp_1
                ld a, #0
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2

            dontChangeSpriteUp:

            inc a
            ld (animationClock), a
        ;;============================;;
    ;;
    ;;Down
    ;;
    sprite42:
        ld a, p_vx(ix)
        cp #0
        jr nz, sprite52
        ld a, p_vy(ix)
        cp #4
        jr nz, sprite52

        ;;======Changes frame==========;;
            ;;
            ;;Checks clock
            ;;
            ld a, (animationClock)
            cp #2
            jr nz, dontChangeSpriteDown
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameDown                    ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerDown_0
                ld a, #1
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2
            ;;
            ;;Frame 2
            ;;
            playerFrameDown:
                ld hl, #_playerDown_1
                ld a, #0
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2

            dontChangeSpriteDown:

            inc a
            ld (animationClock), a
        ;;============================;;
    ;;
    ;;Up right
    ;;
    sprite52:
        ld a, p_vx(ix)
        cp #1
        jr nz, sprite62

        ld a, (scrolling)
        cp #1
        jr z, changeSpriteScrollingUpRigth
            ld a, p_vy(ix)
            cp #-4
            jr nz, sprite62
        changeSpriteScrollingUpRigth:
        ;;======Changes frame==========;;
            ;;
            ;;Checks clock
            ;;
            ld a, (animationClock)
            cp #2
            jr nz, dontChangeSpriteUpRight
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameUpRight                    ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerUpRight_0
                ld a, #1
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2
            ;;
            ;;Frame 2
            ;;
            playerFrameUpRight:
                ld hl, #_playerUpRight_1
                ld a, #0
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2

            dontChangeSpriteUpRight:

            inc a
            ld (animationClock), a
        ;;============================;;
    ;;
    ;;Up left
    ;;
    sprite62:
        ld a, p_vx(ix)
        cp #-1
        jr nz, sprite72
        ld a, p_vy(ix)
        cp #-4
        jr nz, sprite72
        ;;======Changes frame==========;;
            ;;
            ;;Checks clock
            ;;
            ld a, (animationClock)
            cp #2
            jr nz, dontChangeSpriteUpLeft
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameUpLeft                    ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerUpLeft_0
                ld a, #1
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2
            ;;
            ;;Frame 2
            ;;
            playerFrameUpLeft:
                ld hl, #_playerUpLeft_1
                ld a, #0
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2

            dontChangeSpriteUpLeft:

            inc a
            ld (animationClock), a
        ;;============================;;
        
    ;;
    ;;Down right
    ;;
    sprite72:
        ld a, p_vx(ix)
        cp #1
        jr nz, sprite82
        ld a, p_vy(ix)
        cp #4
        jr nz, sprite82

        ;;======Changes frame==========;;
            ;;
            ;;Checks clock
            ;;
            ld a, (animationClock)
            cp #2
            jr nz, dontChangeSpriteDownRight
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameDownRight                    ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerDownRight_0
                ld a, #1
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2
            ;;
            ;;Frame 2
            ;;
            playerFrameDownRight:
                ld hl, #_playerDownRight_1
                ld a, #0
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2

            dontChangeSpriteDownRight:

            inc a
            ld (animationClock), a
        ;;============================;;

    ;;
    ;;Down left
    ;;
    sprite82:
        ld a, p_vx(ix)
        cp #-1
        jr nz, exit_sprite2
        ld a, p_vy(ix)
        cp #4
        jr nz, exit_sprite2
        ;;======Changes frame==========;;
            ;;
            ;;Checks clock
            ;;
            ld a, (animationClock)
            cp #2
            jr nz, dontChangeSpriteDownLeft
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameDownLeft                   ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerDownLeft_0
                ld a, #1
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2
            ;;
            ;;Frame 2
            ;;
            playerFrameDownLeft:
                ld hl, #_playerDownLeft_1
                ld a, #0
                ld (m_playerAnimation), a

                ld a, #0
                ld (animationClock), a

                jp setSprite2

            dontChangeSpriteDownLeft:

            inc a
            ld (animationClock), a
            ret
        ;;============================;;

    setSprite2:
    ld de_sprite_l(ix), l
    ld de_sprite_h(ix), h
    ;;Check 
    exit_sprite2:
ret

player_changeSpriteByFiring:
    ld a, #0
    ld (m_isMoving), a

    ld a, p_vx(ix)
    cp #0
    jr z, dontChange
        ;;Do animation
        ld a, #1
        ld (m_isMoving), a
        jp dontScrolling
    
    dontChange:
    ld a, p_vy(ix)
    cp #0
    jr z, dontChange2
        ;;Do animation
        ld a, #1
        ld (m_isMoving), a

    dontChange2:
    ld a, (scrolling)
    cp #1
    jr nz, dontScrolling
        ld a, #1
        ld (m_isMoving), a
    dontScrolling:

    ;;
    ;;====Right====
    ;;
    sprite1:
        ld a, b_vx(iy)
        cp #2
        jr nz, sprite2
        ld a, b_vy(iy)
        cp #0
        jr nz, sprite2
        ;;== Set bullet's position ==;;
            ld  a, de_x(ix)
            ld  b, de_w(ix)
            add b
            ld  de_x(iy), a
            ld  a, de_y(ix)
            add #6
            ld  de_y(iy), a
        ;;============================;;

        ;;======Changes frame==========;;
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameRight2                     ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerRight_0

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationRight                ;;If you don't move, the sprite doesn't changes
                    ld a, #1
                    ld (m_playerAnimation), a
                dontAnimationRight:
                    jp setSprite
            ;;
            ;;Frame 2
            ;;
            playerFrameRight2:
                ld hl, #_playerRight_1

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationRight2                ;;If you don't move, the sprite doesn't changes
                    ld a, #0
                    ld (m_playerAnimation), a
                dontAnimationRight2:
                    jp setSprite
        ;;============================;;
    ;;
    ;;====Left====
    ;;
    sprite2:

        ld a, b_vx(iy)
        cp #-2
        jr nz, sprite3
        ld a, b_vy(iy)
        cp #0
        jr nz, sprite3

        ;;== Set bullet's position ==;;
            ld  a, de_y(ix)
            add #6
            ld  de_y(iy), a
        ;;============================;;


        ;;======Changes frame==========;;
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameLeft2                     ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerLeft_0

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationLeft                 ;;If you don't move, the sprite doesn't changes
                    ld a, #1
                    ld (m_playerAnimation), a
                dontAnimationLeft:
                    jp setSprite
            ;;
            ;;Frame 2
            ;;
            playerFrameLeft2:
                ld hl, #_playerLeft_1

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationLeft2                ;;If you don't move, the sprite doesn't changes
                    ld a, #0
                    ld (m_playerAnimation), a
                dontAnimationLeft2:
                    jp setSprite
        ;;============================;;
    
    ;;
    ;;====Up====
    ;;
    sprite3:
        ld a, b_vx(iy)
        cp #0
        jr nz, sprite4
        ld a, b_vy(iy)
        cp #-8
        jr nz, sprite4
        ;;== Set bullet's position ==;;
            ld  a, de_x(ix)
            add #2
            ld  de_x(iy), a
            ld  a, de_y(ix)
            dec a
            ld  de_y(iy), a
        ;;============================;;

        ;;======Changes frame==========;;
            ld a, (m_playerAnimation)
            cp #0 
            jr nz, playerFrameUp2               ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerUp_0

                ld a, (m_isMoving)              ;;If you don't move, the sprite doesn't changes
                cp #0
                jr z, dontAnimationUp
                    ld a, #1
                    ld (m_playerAnimation), a
                dontAnimationUp:
                    jp setSprite
            ;;
            ;;Frame 2
            ;;
            playerFrameUp2:
                ld hl, #_playerUp_1

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationUp2          ;;If you don't move, the sprite doesn't changes
                    ld a, #0
                    ld (m_playerAnimation), a
                dontAnimationUp2:
                    jp setSprite
        ;;============================;;

    ;;
    ;;====Down====
    ;;
    sprite4:
        ld a, b_vx(iy)
        cp #0
        jr nz, sprite5
        ld a, b_vy(iy)
        cp #8
        jr nz, sprite5
        ;;== Set bullet's position ==;;
            ld  a, de_y(ix)
            ld  b, de_h(ix)
            add b
            ld  de_y(iy), a
            ld  a, de_x(ix)
            add #2
            ld  de_x(iy), a
        ;;============================;;

        ;;======Changes frame==========;;
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameDown2                     ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerDown_0

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationDown              ;;If you don't move, the sprite doesn't changes
                    ld a, #1
                    ld (m_playerAnimation), a
                dontAnimationDown:
                    jp setSprite
            ;;
            ;;Frame 2
            ;;
            playerFrameDown2:
                ld hl, #_playerDown_1

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationDown2                ;;If you don't move, the sprite doesn't changes
                    ld a, #0
                    ld (m_playerAnimation), a
                dontAnimationDown2:
                    jp setSprite
        ;;============================;;

    ;;
    ;;====Up right====
    ;;
    sprite5:
        ld a, b_vx(iy)
        cp #2
        jr nz, sprite6
        ld a, b_vy(iy)
        cp #-8
        jr nz, sprite6
        ;;== Set bullet's position ==;;
            ld  a, de_x(ix)
            ld  b, de_w(ix)
            add b
            ld  de_x(iy), a
        ;;============================;;

        ;;======Changes frame==========;;
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameUpRight2                     ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerUpRight_0

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationUpRight              ;;If you don't move, the sprite doesn't changes
                    ld a, #1
                    ld (m_playerAnimation), a
                dontAnimationUpRight:
                    jp setSprite
            ;;
            ;;Frame 2
            ;;
            playerFrameUpRight2:
                ld hl, #_playerUpRight_1

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationUpRight2                ;;If you don't move, the sprite doesn't changes
                    ld a, #0
                    ld (m_playerAnimation), a
                dontAnimationUpRight2:
                    jp setSprite
        ;;============================;;
    
    ;;
    ;;====Up left====
    ;;
    sprite6:
        ld a, b_vx(iy)
        cp #-2
        jr nz, sprite7
        ld a, b_vy(iy)
        cp #-8
        jr nz, sprite7

        ;;== Set bullet's position ==;;
            ld  a, de_x(ix)
            dec a
            ld  de_x(iy), a
        ;;============================;;

        ;;======Changes frame==========;;
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameUpLeft2                     ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerUpLeft_0

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationUpLeft            ;;If you don't move, the sprite doesn't changes
                    ld a, #1
                    ld (m_playerAnimation), a
                dontAnimationUpLeft:
                    jp setSprite
            ;;
            ;;Frame 2
            ;;
            playerFrameUpLeft2:
                ld hl, #_playerUpLeft_1

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationUpLeft2                ;;If you don't move, the sprite doesn't changes
                    ld a, #0
                    ld (m_playerAnimation), a
                dontAnimationUpLeft2:
                    jp setSprite
        ;;============================;;
        
    ;;
    ;;====Down right====
    ;;
    sprite7:
        ld a, b_vx(iy)
        cp #2
        jr nz, sprite8
        ld a, b_vy(iy)
        cp #8
        jr nz, sprite8

        ;;== Set bullet's position ==;;
            ld  a, de_x(ix)
            ld  b, de_w(ix)
            add b
            ld  de_x(iy), a
            ld  a, de_y(ix)
            ld  b, de_h(ix)
            add b
            ld  de_y(iy), a
        ;;============================;;

        ;;======Changes frame==========;;
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameDownRight2                     ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerDownRight_0

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationDownRight           ;;If you don't move, the sprite doesn't changes
                    ld a, #1
                    ld (m_playerAnimation), a
                dontAnimationDownRight:
                    jp setSprite
            ;;
            ;;Frame 2
            ;;
            playerFrameDownRight2:
                ld hl, #_playerDownRight_1

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationDownRight2                ;;If you don't move, the sprite doesn't changes
                    ld a, #0
                    ld (m_playerAnimation), a
                dontAnimationDownRight2:
                    jp setSprite
        ;;============================;;

    ;;
    ;;====Down left====
    ;;
    sprite8:
        ld a, b_vx(iy)
        cp #-2
        jr nz, exit_sprite
        ld a, b_vy(iy)
        cp #8
        jr nz, exit_sprite

        ;;== Set bullet's position ==;;
            ld  a, de_x(ix)
            dec a
            ld  de_x(iy), a
            ld  a, de_y(ix)
            ld  b, de_h(ix)
            add b
            ld  de_y(iy), a
        ;;============================;;

        ;;======Changes frame==========;;
            ld a, (m_playerAnimation)                    
            cp #0 
            jr nz, playerFrameDownLeft2                     ;;Check the next frame
            ;;
            ;;Frame 1
            ;;
                ld hl, #_playerDownLeft_0

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationDownLeft          ;;If you don't move, the sprite doesn't changes
                    ld a, #1
                    ld (m_playerAnimation), a
                dontAnimationDownLeft:
                    jp setSprite
            ;;
            ;;Frame 2
            ;;
            playerFrameDownLeft2:
                ld hl, #_playerDownLeft_1

                ld a, (m_isMoving)
                cp #0
                jr z, dontAnimationDownLeft2                ;;If you don't move, the sprite doesn't changes
                    ld a, #0
                    ld (m_playerAnimation), a
                dontAnimationDownLeft2:
                    jp setSprite
        ;;============================;;
        ret

    setSprite:
    ld de_sprite_l(ix), l
    ld de_sprite_h(ix), h
    ;;Check 
    exit_sprite:
ret

;;============================================================================
;; hero_FireBullet: Creates a bullet on the Hero position
;; DESTROYS:
;;       HL, DE, BC, AF
;;============================================================================

player_FireBullet:

    call bullet_New
    call bullet_Copy
    call player_SetPosPlayer

    ld  a, (bullet_aux_x)
    ld  b_vx(iy), a
    ld  a, (bullet_aux_y)
    ld  b_vy(iy), a

not_fire:
ret

;;============================================================================
;; hero_Firelaser: Creates a laser on the Hero position
;; DESTROYS:
;;       HL, DE, BC, AF
;;============================================================================

;;player_Firelaser:
;;
;;    call laser_New
;;    ex de, hl
;;    ld hl, #laser_data
;;    call laser_Copy
;;    call player_SetPosPlayer
;;
;;ret


;;=========================================================================
;;  PLAYER_SETPOSPLAYER: Puts a bullet in the same position as player
;;  INPUTS:
;;      IX -> Pointer to bullet
;;  DESTROYS:
;;      AF
;;=========================================================================

player_SetPosPlayer:   
    push ix

    ld ix, #player_data
    ld a, de_x(ix)
    ld de_x(iy), a
    ld a, de_y(ix)
    ld de_y(iy), a

    pop ix
ret


;;=====================================================
;; player_GETPLAYERDATAIY: returns PLAYER_DATA on IY
;; OUTPUTS:
;;      IY -> Pointer to PLAYER_DATA
;;=====================================================

player_GetPlayerDataIY:

    ld  iy, #player_data

ret

;;=====================================================
;; player_SETPU: Sets a PowerUp to the player
;;=====================================================

player_SetPU:

    ld  hl, #m_has_laser
    ld  (hl), #1

ret

;;====================================================================
;; ENT_COPY: Copy the values of one entity to another 
;; DESTROY: IX, IY
;; INPUT:   
;;     HL -> Origin entity
;;     DE -> Destinatio entity
;;====================================================================

player_Copy:

    ld  bc, #player_data_size        ;;Copy the size of the entity
    ldir 

ret

;;====================================================              
;; player_Hit: Check player's lives and kill him if it's necessary.
;;           comeback to menu
;; DESTROY: HL, IX, DE, A               
;;=====================================================


player_Hit:

    ld  a, p_lf(iy)             ;;A = player lifes
    dec a                       ;;A--
    cp  #0                      ;;If A = 0, you died
    jr  nz, still_alive         ;;Else lifes--

        pop af                  ;;Exit enemy_CheckCollision
        pop af                  ;;Exit enemy_CheckProximities
        pop af                  ;;Exit PUSH AF counter DoForAll
        pop af                  ;;Exit enemy_DoForAll
        jp  player_Die

    still_alive:
    ld  p_lf(iy), a
    pop af                      ;;Exit enemy_CheckCollision
    pop af                      ;;Exit enemy_CheckProximities
    pop af                      ;;Exit PUSH AF counter DoForAll
    pop af                      ;;Exit enemy_DoForAll

    call enemy_DestroyAll
    call gui_DrawTopData
    jp game_SetPlayerDeadState  ;;Stop updating the game

ret 

player_HitByBullet:

    ld  a, p_lf(iy)             ;;A = player lifes
    dec a                       ;;A--
    cp  #0                      ;;If A = 0, you died
    jr  nz, still_alive3         ;;Else lifes--

        pop af                  ;;Exit enemy_CheckCollision
        pop af                  ;;Exit enemy_CheckProximities
        pop af                  ;;Exit PUSH AF counter DoForAll
        pop af                  ;;Exit enemy_DoForAll
        jp  player_Die

    still_alive3:
    ld  p_lf(iy), a
    pop af                      ;;Exit enemy_CheckCollision
    pop af                      ;;Exit enemy_CheckProximities
    pop af                      ;;Exit PUSH AF counter DoForAll
    pop af                      ;;Exit enemy_DoForAll
    push ix                     ;;IX -> enemy_data and IY-> player_data. Enemy_Destroy uses IY to make the operation
    pop iy                      ;;So we do IX <> IY
    call bulletEn_Destroy          ;;Destroy the enemy
    call gui_DrawTopData
    jp game_SetPlayerDeadState  ;;Stop updating the game

ret 

player_HitByObstacle:
    ld  a, p_lf(ix)             ;;A = player lifes
    dec a                       ;;A--
    cp  #0                      ;;If A = 0, you died
    jr  nz, still_alive2         ;;Else lifes--

        pop af                  ;;Exit enemy_CheckCollision
        pop af                  ;;Exit enemy_CheckProximities
        jp  player_Die

    still_alive2:
    ld  p_lf(ix), a
    pop af                      ;;Exit enemy_CheckCollision
    pop af                      ;;Exit enemy_CheckProximities

    call player_SetNewPlayerPosition
    call gui_DrawTopData
    jp game_SetPlayerDeadState  ;;Stop updating the game
ret

player_SetNewPlayerPosition:
    ;;
    ;;The obstacle is in your RIGHT SIDE
    ;;
    ld a, p_bm(ix)
    cp #1
    jr nz, check_LeftCollisionObs
        ld a, de_x(ix)
        cp #18                                         ;;LEFT side screen (18)
        jp m, specialSpawnRight
        jr c, specialSpawnRight
                ld a, de_x(ix)
                add a, #-4
                ld de_x(ix), a
                ret
        specialSpawnRight:                             ;;LEFT side screen(Movement)
            ld a, #170 
            cp de_y(ix)                                ;;DOWN side screen (170)
            jp m, specialSpawnRight2
            jr c, specialSpawnRight2
                ld a, de_y(ix)
                add a, #18
                ld de_y(ix), a
                ret 
        specialSpawnRight2:                            ;;DOWN side screen(Movement)
            ld a, de_y(ix)
            add a, #-18
            ld de_y(ix), a
            ret 

    ;;
    ;;The obstacle is in your LEFT SIDE
    ;;
    check_LeftCollisionObs:
    cp #2
    jr nz, check_DownCollisionObs                       ;;RIGHT side screen (70)
        ld a, #65
        cp de_x(ix)                                    
        jp m, specialSpawnLeft
        jr c, specialSpawnLeft
                ld a, de_x(ix)                        ;;NORMAL
                add a, #4
                ld de_x(ix), a
                ret
        specialSpawnLeft:                             
            ld a, #170 
            cp de_y(ix)                               ;;DOWN side screen (170)
            jp m, specialSpawnLeft2
            jr c, specialSpawnLeft2                    
                ld a, de_y(ix)                        ;;RIGHT side screen(Movement)
                add a, #18
                ld de_y(ix), a
                ret 
        specialSpawnLeft2:                            ;;DOWN side screen(Movement)
            ld a, de_y(ix)
            add a, #-18
            ld de_y(ix), a
            ret 

    ;;
    ;;The obstacle is in your DOWN SIDE
    ;;

    check_DownCollisionObs:
    cp #3
    jr nz, check_UpCollisionObs
        ld a, de_y(ix)
        cp #115                                         ;;UP side screen (85)
        jp m, specialSpawnDown
        jr c, specialSpawnDown
                ld a, de_y(ix)
                add a, #-4                             ;;NORMAL Movement (Move player 8 up)
                ld de_y(ix), a
                ret
        specialSpawnDown:                             
            ld a, #65                                  ;;UP RIGHT side screen (x->65, y->85)
            cp de_x(ix)                               
            jp m, specialSpawnDown2
            jr c, specialSpawnDown2
                ld a, de_x(ix)                         ;;If you aren't at down-right -> Move right
                add a, #4
                ld de_x(ix), a
                ret 
        specialSpawnDown2:
                ld a, de_x(ix)                         
                add a, #-4
                ld de_x(ix), a
                ret 
       ret

    ;;
    ;;The obstacle is in your UP SIDE
    ;;
    check_UpCollisionObs:
        ld a, #165
        cp de_y(ix)                                    ;;DOWN side screen (165)
        jp m, specialSpawnUp
        jr c, specialSpawnUp
                ld a, de_y(ix)
                add a, #8                              ;;NORMAL Movement (Move player 8 down)
                ld de_y(ix), a
                ret
        specialSpawnUp:                             
            ld a, #65                                  ;;DOWN RIGHT side screen (x->65, y->165)
            cp de_x(ix)                               
            jp m, specialSpawnUp2
            jr c, specialSpawnUp2
                ld a, de_x(ix)                         ;;If you aren't at down-right -> Move right
                add a, #4
                ld de_x(ix), a
                ret 
        specialSpawnUp2:
                ld a, de_x(ix)                         
                add a, #-4
                ld de_x(ix), a
                ret 
ret


;;====================================================              
;; player_Die: reset and clear the enemy vector and the hero 
;;           comeback to menu
;; DESTROY: HL, IX, DE, A               
;;=====================================================

player_Die:
    call buffer_clearFront
    ld ix, #player_data 
    push ix 
    pop de
    ld hl, #player_backup
    call player_Copy

    call enemy_DestroyEveryEnemy        ;;Destroy every enemy
    call bullet_DestroyEveryBullet      ;;Destroy every bullet
    call obstacle_DestroyEveryObstacle  ;;Destroy every obstacle
    call bulletEn_DestroyEveryBullet
    call pu_DestroyAllPU

    ld  a, #1
    ld  (menu_state_game), a
    ld  a, #1
    ld  (dead_state_game), a

    ;; === Reset map === ;;
    ld hl, #Level11
    ld  (m_next_element), hl        ;;Now we point to the first elemento of the next level

    ld  a, #10                       ;;RegionCounter = 10
    ld  (regionCounter), a          ;;|

    ld  a, #0                       ;;ScrollsCounter = 0
    ld  (scrollsCounter), a         ;;|

    ; === Reset levels ===
    ld a, #0
    ld (level2), a
    ld a, #0
    ld (level3), a

    call buffer_clearBack
    call buffer_switch

ret


player_Win:
    call buffer_clearFront
    ld ix, #player_data 
    push ix 
    pop de
    ld hl, #player_backup
    call player_Copy

    call enemy_DestroyEveryEnemy        ;;Destroy every enemy
    call bullet_DestroyEveryBullet      ;;Destroy every bullet
    call obstacle_DestroyEveryObstacle  ;;Destroy every obstacle
    call bulletEn_DestroyEveryBullet
    call pu_DestroyAllPU

    ld  a, #1
    ld  (menu_state_game), a
    ld  a, #0
    ld  (dead_state_game), a
    ld  a, #1
    ld  (winner_win_state), a

    ;; === Reset map === ;;
    ld hl, #Level11
    ld  (m_next_element), hl        ;;Now we point to the first elemento of the next level

    ld  a, #10                       ;;RegionCounter = 10
    ld  (regionCounter), a          ;;|

    ld  a, #0                       ;;ScrollsCounter = 0
    ld  (scrollsCounter), a         ;;|

    call buffer_clearBack
    call buffer_switch

ret