;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "powerUp.h.s"
.include "render.h.s"
.include "bullet.h.s"
.include "entity.h.s"
.include "enemy.h.s"
.include "player.h.s"
.include "main.h.s"
.include "sound.h.s"

;;LOCAL VARIABLES
m_next_pu: .dw pu_vector       ;;Pointer of the memory position of the next entity 
m_num_pu: .db 00                   ;;Counter to see how many entities we have created
pu_mem: .dw 0x0000

;;============================== CONSTANTS =======================================;;

;;== pus ==;;
k_max_num_pu = 10                      ;;Constant for the maximum number of pus
k_pu_size = pu_vector0_size            ;;Dynamic size of pu
m_pu_lft = 40                          ;;Life cycles of a pu
 
;;== pu vector ==;;
pu_vector:
DefineNPU pu_vector, k_max_num_pu 

;; == Level With Power Ups == ;;
level2: .db 0

;;== Default pu for new instances, i.e. the default constructor ==;;
DefinePU pu_data, #50, #40, 0x00, 0x00, 0x04, 0x10, _pu_sprite, 0x00, 0x00, 0x00, 0x00, pu_CheckLFT, m_pu_lft, 0     ;;We have a new variable
                                                                                       ;;_ltf (lifetime)

;;========================================================================================================;;
;;                                                                                                        ;;
;;                                         PU PRIVATE METHODS                                             ;;
;;                                                                                                        ;;
;;========================================================================================================;;

pu_New:

    ld  a, (m_num_pu)
    inc a                          ;;A++
    ld (m_num_pu), a               ;;<>  

    ld hl, (m_next_pu)             ;;HL = Pointer to the last active pu    
    ld bc, #k_pu_size              ;;BC = pu's size
    add hl, bc                     ;;HL += BC -> the position of the next new pu
    ld (m_next_pu), hl             ;;NEXT_ENTITY = HL (last active pu)
    or  a                          ;;To put Carry to 0 (a or a = 0)
    sbc hl, bc                     ;;HL -= BC -> to get the pointer to the current entity

ret

;;============================================================
;; pu_COPY: Copy the values of one pu to another 
;; DESTROY: IX, IY
;; INPUT:   HL -> Origin pu
;;          DE -> Destinatio pu
;;          BC -> Size of the copy
;;============================================================

pu_Copy:

    ld (pu_mem), de            ;;
    ld iy, (pu_mem)
    ld  bc, #k_pu_size         ;;Copy the size of the entity
    ldir 

ret

;;============================================================
;; pu_DESTROY: Deletes a pu from the memory
;; INPUT:   
;;    IX -> Pointer to THIS pu
;; DESTROYS:
;;    AF, HL, DE, BC
;;============================================================

pu_Destroy:
    call ren_Clear_FrontBack

    ld  a, (m_num_pu)
    cp  #0
    jr  z, exit_destroy

        call pu_TurnBackNextPU               ;;M_NEXT_pu one position back
        call pu_DecreaseNumPUs               ;;M_NUM_ENEMIES--
        
        push ix
        pop de

        ld  a, h                             ;;Check if the last pu is the same as THIS
        cp  d                                ;;|
        jr nz, make_copy                     ;;|

        ld  a, l                             ;;|
        cp  e                                ;;|
        jr nz, make_copy                     ;;|

        call pu_SetDeadState                 ;;In other case, don't make a Copy,
        ret	                                 ;;it's the same direction
                                             ;;only set this position (the last) to dead
        make_copy:

        ld  bc, #k_pu_size                   ;;\Copy the last pu of the vector
        ldir                                 ;;\to the position of THIS pu that we want to destroy
        call pu_SetDeadState                 
    
exit_destroy:
ret

;;=================================================              
;;  pu_DESTROYALLPU: Destroys all the enemies
;;  DESTROYS:
;;      AF, BC, HL
;;=================================================

pu_DestroyAllPU:

    ld  hl, #pu_Destroy
    jp pu_DoForAll

;;=======================================================================
;; pu_TURNBACKNEXTpu: Set back next pu pointer
;; DETROY: AF, HL, BC 
;;=======================================================================

pu_TurnBackNextPU:

    ld  hl, (m_next_pu)                 ;;HL = Pointer to the last active pu                                                          
    ld  bc, #k_pu_size                  
    or a 
    sbc hl, bc
    ld  (m_next_pu), hl  

ret

;;=======================================================================
;; pu_DECREASENUMpuS: Decrease the number of pus 
;; DETROY: AF 
;;=======================================================================

pu_DecreaseNumPUs:

    ld  a, (m_num_pu)                   ;;A = number of active pus
    dec a                                   ;;A --
    ld  (m_num_pu), a                   ;;<>
ret

;;=======================================================================
;; pu_SETDEADSTATE: Set the last pu's dead state to 1
;; DETROY: A
;;=======================================================================

pu_SetDeadState:
    push ix

        ld ix, (m_next_pu)
        ld a, #1
        ld pu_dead(ix), a
        
    pop ix 
ret

;;=======================================================================
;; pu_DOFORALL: Update, draw and clear to all pus
;; INPUT:  HL-> Function 
;;=======================================================================

pu_DoForAll:

    ld a, (m_num_pu)               ;;Loads the number of the entities 
    cp #0
    jr z, not_dfa  

    ld ix, #pu_vector              ;;Loads the first entity of the vector on IX
    ld (method), hl                ;;HL = function parameter
    
    loop:

        push af                    ;;Push a on the stack to not lose it

        ld a, pu_dead(ix)
        cp #1
        jr z, dead

        method = . + 1             ;;Current position + 1 = the function of the next call
        call pu_CheckLFT           ;;We call the method we have in hl 

        dead:

        pop af

        ld bc, #k_pu_size          ;;Move M_NEXT_pu pointer to the next position by the pus size
        add ix, bc              
        dec a            
    jr nz, loop                    ;;Next iteration

not_dfa:
ret

;;===========================================================
;; pu_CHECKLFT (*): 
;;
;; (*) -> Tail recursion
;;===========================================================

pu_CheckLFT:

    call pu_PlayerCollision

    ld   a, pu_lft(ix)               ;;Check the lifetime time of ix pu
    cp   #0                          ;;|
    jr   z, pu_die                   ;;\ If it's time to die, then stop updating

        dec a   
        ld  pu_lft(ix), a
        ret
        
    pu_die:
    jp pu_Destroy


pu_PlayerCollision:
    ld  iy, #player_data
    ;;
    ;;Check collision in X axis
    ;;
    call check_entities_collision_x

    ;;
    ;;Make if statement
    ;;
    jr z, no_collision
    jp m, no_collision

    ;;
    ;;If i collide in x, then i check y
    ;;
    call check_entities_collision_y
    jr z, no_collision
    jp m, no_collision

    ;;
    ;;If there is collision
    ;;
    call bullet_ActivatePU
    jp  pu_die 
    
    no_collision:
ret

;;===================================================
;; pu_MOVESCROLL: Move pus with scroll and kill it 
;; INPUT:
;;      IX -> Pointer to PU
;; DESTROYS:
;;      AF, BC
;;===================================================

pu_MoveScroll:

    ;;
    ;;Check down side screen collision before move the PU
    ;;
    ld  a, #200-10             ;;A = 190 (Down side screen - (Height + VX of PU)     
    ld  b, de_y(ix)            ;;B = DE_Y (Y position of the PU)
    cp  b                      ;;A -= B
    jr  z, kill_pu             ;;If z = 1, then destroy the bullet
    jp  c, kill_pu             ;;(200 - 10) - DE_X -> C (Example: RESULT = FA = -6 = 250), so 

        ld pu_vy(ix), #4       ;;In
        call ent_Move          ;;Move
        ld pu_vy(ix), #0       ;;Reset velocity for 
        ret

    kill_pu:
    jp pu_Destroy


;;==================================================================              
;;  pu_GETPUSIZE: Returns the size of the pu's vector
;;  OUTPUT:
;;      BC -> Size of an pu
;;  DESTROYS:
;;      BC
;;==================================================================

pu_GetPUSize:

    ld  a, #k_pu_size

ret


;;==============================================================================================              
;;  pu_TRYTOSPAWNPU: Calculates a random probabilty of spawning a PowerUp on the Enemy position
;;  INPUT:
;;      DE -> (X, Y) Dead Enemy position
;;       L -> Entropy byte to generate the pseudo random sequence
;;  DESTROYS:
;;      AF, BC, HL, DE
;;===============================================================================================

pu_TryToSpawnPU:

    ld a, (level2)               ;;if its not level 3 or more we cant spawn enemies
    cp #0
    jr z, exit_ttspu
    
	call	cpct_getRandom_lcg_u8_asm		;; We fetch a rnd [0, 7]
	ld		a,	l							;; A = L = Rnd Num
	ld		b,	#0x03						;; Mascara para coger 2 bits
	and		b								;; A = [0, 7]
	cp	#0									;; Check against 0, (12'5% of spawning)
	jp  nz, exit_ttspu

    push iy                                 ;;Push IY on the stack, because IY is pointing to Enemy
    push de                                 ;;Save DE = Position of the enemy that is leaving the Power Up
                                            
    call pu_New					            
    ex de, hl
    ld hl, #pu_data       
    call pu_Copy

    pop de                                  ;;Recovering DE
        
    ld  de_x(iy), d                         ;;PowerUp position = Dead Enemy position
    ld  de_y(iy), e                         ;;\
    
    pop iy                                  ;;IY = Pointer to Enemy that is going to die

exit_ttspu:
ret

pu_Spawn:

    call pu_New
    ex de, hl
    ld  hl, #pu_data
    call pu_Copy

ret
