;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of BringRGBack: An Amstrad CPC Game 
;;  Copyright (C) 2018 Ivan Palacios Verano / Edward Thomas Owen Nolla / Javier Richarte Gras
;;
;;  BringRGBack is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  BringRGBack is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with BringRGBack.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.area _DATA
.area _CODE

.include "main.h.s"
.include "sound.h.s"

tempo: .db #06 
tempo_value_default: .db  #6
instrument: .db #00

stopMusic:

    call cpct_akp_stop_asm

ret


loadLevelMusic:

    ld a, #5
    ld (tempo_value_default), a
    ld de, #_level_theme
    call cpct_akp_musicInit_asm

ret

;;====================================================================
;;  LOADSOUNDEFFECTS
;;  It loads the whole sound effects package and sets it ready to play
;;====================================================================
loadSoundEffects:

      ld de, #_sfx_theme
      call cpct_akp_SFXInit_asm

ret

;;INPUTS - A: Instrument
setSFXInstrument::

    ld (instrument), a

ret
;;====================================================================
;;  PLAYSOUNDEFFECT
;;  It plays a certain sound effect
;;  INPUTS: L - Sound effect to be played
;;====================================================================
playSoundEffect:
    
    ld a, (instrument)
    cp #0
    jr z, no_play

    ld l, a
    ld h, #15 ;;Volume of the sound effect
    ld e, #100 ;;Note to be played - CO is 0
    ld d, #0  ;;Speed as original sound
    ld bc, #0x0000 ;;I think this mustn't be touched
    ld a, #0b00000100 ;;Play on channel 3

    call cpct_akp_SFXPlay_asm

    ld a, #0
    ld (instrument), a

    no_play:

ret


;;=============================================================
;;  INTERRUPTION HANDLER
;;  It plays the music as it must be with no problems of tempo
;;=============================================================
playMusic:
    ex af, af'
    exx
    push af
    push bc
    push de
    push hl
    push iy

    ld a, (tempo)
    dec a
    ld (tempo), a
    jr nz, return

    call cpct_akp_musicPlay_asm
    call playSoundEffect
    
    ld a, (tempo_value_default)
    ld (tempo), a

    return:

    pop iy
    pop hl
    pop de
    pop bc
    pop af
    exx
    ex af, af'

ret

sound_PlayShot:
    push af
        ld a, #3
        call setSFXInstrument
        call playSoundEffect
    pop af
ret

sound_PlayHitDead:
    push af
        ld a, #4
        call setSFXInstrument
        call playSoundEffect
    pop af
ret

sound_PlayDamage:
    push af
        ld a, #5
        call setSFXInstrument
        call playSoundEffect
    pop af
ret

sound_PlayPowerUp:
    push af
        ld a, #6
        call setSFXInstrument
        call playSoundEffect
    pop af
ret